module CalcLike
 use CMB_Cls
use cmbtypes
 use cmbdata
 use mpk
 use Random
 use settings
 use ParamDef
use mcrecsp
use effdust_dh
use prepagnspec
use compphotfnu
!use calcX2_pmax 
use bestsed
use snovae
 use WeakLen
 use Lya
 implicit none

 logical :: Use_Age_Tophat_Prior = .true.
 logical :: Use_CMB = .true.
 logical :: Use_BBN = .false.
 logical :: Use_Clusters = .false.
  
 integer :: H0_min = 40, H0_max = 100
 real :: Omk_min = -0.3, Omk_max = 0.3
 real :: Use_min_zre = 0
 integer :: Age_min = 10, Age_max = 20
 real :: Temperature  = 1

 


contains

  function GenericLikelihoodFunction(Params)



    INTEGER, PARAMETER :: lambdamax = 1221

    !TRANSITION VARIABLES (AA)
    REAL(KIND=8), DIMENSION(lambdamax) :: outflux, boutflux, coutflux
    ! from effdust_dh.f90
    REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: lam, fsed, fsedp
    INTEGER :: nlam
    REAL(KIND=8) :: Lbol0
    ! from compphotfnu.f90
    REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: fnuout
    !END TV (AA)
    

   
    type(ParamSet)  Params 
    real :: GenericLikelihoodFunction
    INTEGER :: i,imod,numP
    real(KIND=8), dimension(4) :: cresppar 
    real(KIND=8), dimension(6) :: effpar

    real(KIND=8)  metall,tauold,ageold,taunew,agenew,burst,ndf
    real(KIND=8) UVbump,crpar,delta,A_ySP,f_V,alpha,agnL
    integer, parameter :: maxpar=28
    integer, parameter :: nmax = 2
    real waveUV,FWHMbump
    real(KIND=8) :: Mgal,Mstar,SFR,age_M,age_D4000,Lbol,Ldust
    real(KIND=8) :: A_FUV,A_V,mean_bump,width_bump,ampl_bump
    real(KIND=8) A_V_ySP, fl_emlin,fl_abslin,minimX2,chidef
    real(KIND=8) expMgal,ratage,ratagetau
    real(KIND=8), dimension(nmax) :: X2


CHARACTER(LEN=80) :: mycrecsp,myfiltatt,mydhmod,myspeclin


! parameterization used:
!stellar population parameters 
   metall=Params%P(1)
   tauold=Params%P(2)
   ageold=Params%P(3)
   taunew=Params%P(4)
   agenew=Params%P(5)
   burst=Params%P(6)


   ratage=ageold/agenew*0.9  ! this way no cycle, this constraint has to be added as a prior not like it is right now! 
   ratagetau=agenew/taunew*1.1 ! this way no cycle, this constraint has to be added as a prior not like it is right now!

!  dust attenuation and IR emission parameters
   UVbump=Params%P(7)
   crpar=Params%P(8)
   delta=Params%P(9)
   A_ySP=Params%P(10)
   f_V=Params%P(11)
   alpha=Params%P(12)
   agnL=Params%P(13)
   Mgal=Params%P(14)
  

    call stellpop(Params,CigVar%age,CigVar%mstar,CigVar%flux,CigVar%dage,CigVar%d4000,&
         CigVar%lambda,CigVar%nage,CigVar%iagemax,outflux,boutflux,coutflux,cresppar,ratage,ratagetau)


  
    waveUV=2175.0
    FWHMbump=300.0



    call dust(Params,CigVar%nirmod,CigVar%irid,CigVar%irslope,CigVar%nirlam,CigVar%irflux,&
         CigVar%irlam,CigVar%nhdlam,CigVar%hdlam,CigVar%hdflux,CigVar%nlambdalin,CigVar%lambdalin,&
         CigVar%emlin,CigVar%abslin,CigVar%nrange,CigVar%lambdarange,outflux,boutflux,&
         coutflux,CigVar%lambda,nlam,lam,fsed,fsedp,effpar,Lbol0)

   

    call theoryfnu(CigVar%nz,CigVar%z,CigVar%nfilters,CigVar%fluxfilt,CigVar%mmax,CigVar%m_IR,&
         CigVar%minlam,CigVar%area,CigVar%trans,CigVar%lamfilt,&
         CigVar%typetrans,CigVar%mnlamfilt,CigVar%ifilt,CigVar%meanlam,nlam,lam,fsed,fsedp,Lbol0,&
         fnuout,CigVar%H0,CigVar%Omega_M,CigVar%Omega_L)

 

! compute the n.d.f and assess the need of a sufficient number of data 

    ndf = real(COUNT(CigVar%fobs > tol) - 1)
 
   if (ndf .le. CigVar%numP) then
      call DoAbort('calclike.f90: too many free parameters! Remember Von Neumann: "With four&
           parameters I can fit an elephant, and with five I can make him wiggle his trunk."')
endif


    expMgal=10.0**Mgal
  
    do imod = 1, nmax
       
       X2(imod) = SUM((expMgal*fnuout(:,imod)-CigVar%fobs)**2/CigVar%dfobs**2,MASK=(CigVar%dfobs > tol))/ndf!+(agenew-1.8)**2/0.04
  
   enddo
 

!   minimX2 = MAX(MINVAL(X2, X2 > -tol), 0.)
! only consider the dust model, otherwise it messes up the mcmc
   minimX2=X2(2) 

   chidef = minimX2

    
!derived parameters; 
     Mstar=log10(cresppar(1))+Mgal
     SFR=log10(cresppar(2))+Mgal
     age_M=cresppar(3)
     age_D4000=cresppar(4)
     Lbol=log10(effpar(1))+Mgal
     Ldust=log10(effpar(2))+Mgal
     A_FUV=effpar(3)
     A_V=effpar(4)
     mean_bump=effpar(5)
     width_bump=effpar(6)   


Params%P(15)=Mstar
Params%P(16)=SFR
Params%P(17)=age_M
Params%P(18)=age_D4000
Params%P(19)=Lbol
Params%P(20)=Ldust
Params%P(21)=A_FUV
Params%P(22)=A_V
Params%P(23)=mean_bump
Params%P(24)=width_bump
!AA add the log of the luminosity fraction from dust over star
Params%P(25)=log10((1.-Params%P(13))*effpar(2)/(effpar(1)-effpar(2)))



   !Used when you want to plug in your own CMB-independent likelihood function:
   !set generic_mcmc=.true. in settings.f90, then write function here returning -Ln(Likelihood)
   !Parameter array is Params%P
 
! this is the new Likelihod for cigale
GenericLikelihoodFunction=chidef/2.d0

   ! GenericLikelihoodFunction = greatLike(Params%P)
  !  GenericLikelihoodFunction = LogZero 
   ! call MpiStop('GenericLikelihoodFunction: need to write this function!')
 !   GenericLikelihoodFunction=0

  end function GenericLikelihoodFunction

  
  function GetLogPrior(CMB, Info) !Get -Ln(Prior)
    real GetLogPrior
    real Age
    Type (CMBParams) CMB
    Type(ParamSetInfo) Info

    GetLogPrior = logZero
 
    if (.not. generic_mcmc) then
     if (CMB%H0 < H0_min .or. CMB%H0 > H0_max) return
     if (CMB%omk < Omk_min .or. CMB%omk > Omk_max .or. CMB%Omv < 0) return
     if (CMB%zre < Use_min_zre) return

     Age = GetAge(CMB, Info)
      !This sets up parameters in CAMB, so do not delete unless you are careful!
 
     if (Use_Age_Tophat_Prior .and. (Age < Age_min .or. Age > Age_max) .or. Age < 0) return
    
    end if
    GetLogPrior = 0
 
  end function GetLogPrior

  function GetLogLike(Params) !Get -Ln(Likelihood)
    type(ParamSet)  Params 
    Type (CMBParams) CMB
    real GetLogLike
    real dum(1,1)
 
    if (any(Params%P > Scales%PMax) .or. any(Params%P < Scales%PMin)) then
       GetLogLike = logZero
        return
    end if

    if (generic_mcmc) then
        GetLogLike = GenericLikelihoodFunction(Params) 
        if (GetLogLike /= LogZero) GetLogLike = GetLogLike/Temperature

    else

     call ParamsToCMBParams(Params%P,CMB)

     GetLogLike = GetLogLikePost(CMB, Params%Info,dum,.false.)
    end if 
   end function GetLogLike

    
  function GetLogLikePost(CMB, Info, inCls, HasCls) 
    real GetLogLikePost
    Type (CMBParams) CMB
    Type(ParamSetInfo) Info
    real, intent(in):: inCls(:,:)
    logical, intent(in) :: HasCls
    real acl(lmax,num_cls_tot)
    integer error

    if (generic_mcmc) stop 'GetLogLikePost: not supported for generic'

    GetLogLikePost  = GetLogPrior(CMB, Info)
    if ( GetLogLikePost >= logZero) then
       GetLogLikePost = logZero
       
    else 
       GetLogLikePost = GetLogLikePost + sum(CMB%nuisance(1:nuisance_params_used)**2)/2
          !Unit Gaussian prior on all nuisance parameters
       if (Use_BBN) GetLogLikePost = GetLogLikePost + (CMB%ombh2 - 0.022)**2/(2*0.002**2) 
          !I'm using increased error bars here
   
       if (Use_CMB .or. Use_LSS) then
          if (HasCls) then
           acl = inCls
           error =0
          else
           call GetCls(CMB, Info, acl, error)
          end if
         if (error /= 0) then
          GetLogLikePost = logZero 
         else
          if (Use_CMB) GetLogLikePost = &
            CMBLnLike(acl, CMB%norm(norm_freq_ix:norm_freq_ix+num_freq_params-1),CMB%nuisance) + GetLogLikePost
          if (Use_mpk) GetLogLikePost = GetLogLikePost + LSSLnLike(CMB, Info%theory)
          if (Use_WeakLen) GetLogLikePost = GetLogLikePost + WeakLenLnLike(CMB, Info%theory)     
          if (Use_Lya) GetLogLikePost = GetLogLikePost +  LSS_Lyalike(CMB, Info%Theory)
          if ( GetLogLikePost >= logZero) then
            GetLogLikePost = logZero
          end if
         end if
         if (Use_SN .and. GetLogLikePost /= logZero ) then
            if (Info%Theory%SN_loglike /= 0) then
             GetLogLikePost = GetLogLikePost + Info%Theory%SN_loglike
            else
             GetLogLikePost = GetLogLikePost + SN_LnLike(CMB)
            end if
               !Assume computed only every time hard parameters change
         end if
         if (Use_BAO .and. GetLogLikePost /= logZero ) then
            if (Info%Theory%BAO_loglike /= 0) then
             GetLogLikePost = GetLogLikePost + Info%Theory%BAO_loglike
            else
             GetLogLikePost = GetLogLikePost + BAO_LnLike(CMB)
            end if
               !Assume computed only every time hard parameters change
         end if
         if (Use_HST .and. GetLogLikePost /= logZero) then
            if (Info%Theory%HST_loglike /= 0) then
             GetLogLikePost = GetLogLikePost + Info%Theory%HST_loglike
            else
             GetLogLikePost = GetLogLikePost + HST_LnLike(CMB)
            end if
           !!Old: GetLogLikePost = GetLogLikePost + (CMB%H0 - 72)**2/(2*8**2)  !HST 
         end if  
     
         else
           if (Use_SN) GetLogLikePost = GetLogLikePost + SN_LnLike(CMB)
           if (Use_BAO) GetLogLikePost = GetLogLikePost + BAO_LnLike(CMB)
           if (Use_HST) GetLogLikePost = GetLogLikePost + HST_LnLike(CMB)           
         end if
         
     
      if (Use_Clusters .and. GetLogLikePost /= LogZero) then
          GetLogLikePost = GetLogLikePost + &
                 (Info%Theory%Sigma_8-0.9)**2/(2*0.05**2)
          stop 'Write your cluster prior in calclike.f90 first!'
      end if

     if (GetLogLikePost /= LogZero) GetLogLikePost = GetLogLikePost/Temperature
   

    end if

  end function GetLogLikePost

end module CalcLike
