module compphotfnu

! Author: Stefan Noll (2009) 
! Computes filter fluxes in mJy of input spectra

use ParamDef
contains

subroutine theoryfnu(nz,z,nfilters,fluxfilt,mmax,m_IR,minlam,area,trans,lamfilt, &
     typetrans,mnlamfilt,ifilt,meanlam,nlam,lam,fsed,fsedp,Lbol0,fnuout,H0,Omega_M,Omega_L)

IMPLICIT NONE

!INPUT (AA)
INTEGER, INTENT(IN) :: nz, nfilters, m_IR
REAL (KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(IN) ::z,area,meanlam
REAL(KIND=8), DIMENSION(:), ALLOCATABLE,  INTENT(IN):: fluxfilt
REAL (KIND=8), DIMENSION(:,:), ALLOCATABLE, INTENT(IN) :: trans, lamfilt
REAL (KIND=8), INTENT(IN) :: minlam,H0,Omega_M,Omega_L
INTEGER, DIMENSION(:), ALLOCATABLE, INTENT(IN) :: typetrans, mnlamfilt,ifilt
! from effdust_dh
REAL(KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(IN) :: lam, fsed, fsedp
REAL(KIND=8) :: Lbol0
INTEGER, INTENT(IN) :: nlam
!END I (AA)

!OUTPUT (AA)
REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: fnuout
!END O (AA)

REAL (KIND=8), DIMENSION(:), ALLOCATABLE ::fluxfilt_temp
CHARACTER(LEN=*), PARAMETER :: suffix = 'fnu' ! OUTPUT!
CHARACTER(LEN=*), PARAMETER :: cosmofile = 'cosmology.dat' ! INPUT! 
INTEGER, PARAMETER :: nmaxlamfilt = 1500 ! Check 'filterfile'!
INTEGER, PARAMETER :: nzmax = 1000 ! Check 'zdata' if '*.dat'!
REAL(KIND=8), PARAMETER :: c_dist = 1d-5 ! Mpc -> 10 pc
REAL(KIND=8), PARAMETER :: lam_Lya = 1216. ! in A
REAL(KIND=8), PARAMETER :: Lsol = 3.827d26 ! in W
REAL(KIND=8), PARAMETER :: meanlam_IR = 1d6 ! in A (arbitrary)
REAL(KIND=8), PARAMETER :: c_area = 1.1967d36 ! surface at 10 pc in m^2
REAL(KIND=8), PARAMETER :: c_flamfnu = 1d19 ! W/(m^2*A) -> mJy
REAL(KIND=8), PARAMETER :: c = 2.99792458d8 ! light speed in m/s
REAL(KIND=8), PARAMETER :: tol = 1d-12

CHARACTER(LEN=80) :: filterfile, calibfile, filtoutpar
CHARACTER(LEN=80) :: outfilelist, zdata, photdat, bayespar, myfile1,myfile2
CHARACTER(LEN=80) :: specfile, dir, spec, label
CHARACTER(LEN=160) :: header, sppar, spval, attpar, attval
CHARACTER(LEN=800) :: fmt, fnupar, allpar 
CHARACTER(LEN=32) :: ID
CHARACTER(LEN=8) :: cdummy, filtlab0, zc
CHARACTER(LEN=4) :: formz
CHARACTER(LEN=1) :: sharp, sign
!LOGICAL :: fl_zdata
INTEGER :: h,  i, idummy, j, ifilt0, m, mmax, zacc,  nval
INTEGER :: nfilt, nobj, iz, minz, nfiles, nvalidfiles, ifiles, nvfile, n, num
INTEGER :: lspace, rspace, ndir, posdot, posuscore
INTEGER :: ncomp, nsppar, nattpar, nattmod, nallpar, nallmod, npar, nmod 
!INTEGER :: status_open, status_read
REAL(KIND=8), DIMENSION(3) :: zpar 
REAL(KIND=8) :: rdummy, Ldust, fluxfilt_IR, fnu0

CHARACTER(LEN=8), DIMENSION(:), ALLOCATABLE :: filtlab
CHARACTER(LEN=80), DIMENSION(:), ALLOCATABLE :: fnufile
INTEGER, DIMENSION(:), ALLOCATABLE :: nlamfilt
INTEGER, DIMENSION(:), ALLOCATABLE :: intz, sintz
REAL(KIND=8), DIMENSION(:), ALLOCATABLE ::  tempz, d_L
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: flux, zlam, zflux
REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: fluxcorr
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: czflux
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: fnu





! *** Luminosity distance in Mpc for z ***

ALLOCATE (d_L(nz))
iz =1
!DO iz = 1, nz
d_L(iz) = LumDist(z(iz), H0, Omega_M, Omega_L)
!   PRINT*, "LumDist1: ", iz, z(iz), d_L(iz)
IF (d_L(iz) < c_dist) d_L(iz) = c_dist
!   PRINT*, "LumDist2: ", iz, z(iz), d_L(iz)

!END DO


! *** Prepare reading of spectra ***

ALLOCATE (fnu(mmax))

ALLOCATE (fnuout(mmax,2))

nfiles=1
nvalidfiles=1

nvfile = 1 
nallpar = 0
nallmod = 0


ifiles=1





!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
nsppar = 10                 !
nattpar = 16                !
nattmod = 2                 ! 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

npar = nsppar + nattpar + 1
nallpar = nallpar + npar
IF (nallpar /= npar * nvfile) THEN
   PRINT*, 'Unexpected number of parameters! -> Exit loop!'
   STOP
!   EXIT
END IF

 

IF (nvfile == 1) nmod = nattmod * nvalidfiles
  
   nallmod = nallmod + nattmod
   IF (nallmod /= (nmod / nvalidfiles) * nvfile) THEN
      PRINT*, 'Unexpected number of models! -> Wrong output header!'
   END IF



   ! Read spectra, compute filter fluxes, and write files
   
   ALLOCATE (flux(nlam))
   ALLOCATE (zlam(nlam), zflux(nlam))
   ALLOCATE (fluxcorr(nlam,iz))
   ALLOCATE (czflux(nlam))
    


   ALLOCATE(fluxfilt_temp(nfilters))
!   fluxfilt_temp=fluxfilt
 



   DO n = 1, nattmod
      
      num = (ifiles - 1) * nattmod + n 

      IF ( n == 1 ) THEN 
         flux=fsedp
 
      ELSE
         flux=fsed

        


      ENDIF


IF (m_IR > 0) THEN
         Ldust=Lbol0
         endif

    

      iz = 1

      zlam = lam * (1 + z(iz))
      zflux = flux / (1 + z(iz)) 
     
      ! Consider absorption by the IGM if filters are affected
      IF (lam_Lya * (1 + z(iz)) >= minlam) THEN
         IF (n == 1) CALL CorrIGM (zlam, z(iz), fluxcorr(:,iz))
         czflux = zflux * fluxcorr(:,iz)
      ELSE
         czflux = zflux
      END IF


      
     
      CALL calcfluxfilt (mnlamfilt, typetrans, lamfilt, trans, &
           area, zlam, czflux, fluxfilt_temp)

    
      IF (m_IR > 0) &
           fluxfilt_IR = (Ldust * Lsol / meanlam_IR) / (1 + z(iz))

      DO m = 1, mmax
         IF (m == m_IR) THEN
            fnu0 = (c_flamfnu / c) * (meanlam_IR ** 2) * &
                 fluxfilt_IR / c_area
         ELSE
            fnu0 = (c_flamfnu / c) * (meanlam(ifilt(m)) ** 2) * &
                 fluxfilt_temp(ifilt(m)) / c_area

               
         END IF

         fnu(m) = fnu0 / (d_L(iz) / c_dist) ** 2

      

     
      

         ENDDO
         
         
      fnuout(:,n)=fnu(:)
 
     

   END DO
      


   DEALLOCATE (flux, zlam, zflux, fluxcorr, czflux,fluxfilt_temp)



CONTAINS

  SUBROUTINE calcfluxfilt (nlamfilt, typetrans, lamfilt, trans, area, &
       lam, flux, fluxfilt)
    
    ! Calculation of the flux in the ith filter
    
    IMPLICIT NONE
    INTEGER, DIMENSION(:), INTENT(IN) :: nlamfilt, typetrans
    REAL(KIND=8), DIMENSION(:,:), INTENT(IN) :: lamfilt, trans
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: lam, flux
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: area
    REAL(KIND=8), DIMENSION(:), INTENT(OUT) :: fluxfilt
    INTEGER :: nfilt, nlam, i, j, k  
    REAL(KIND=8) :: fluxtrans, laminf, lamsup
    REAL(KIND=8) :: fluxinf, fluxsup, transinf, transsup


  
    nfilt = SIZE(nlamfilt)
    nlam = SIZE(lam)


    DO i = 1, nfilt
 
       IF (nlamfilt(i) < 1) THEN
          fluxfilt(i) = 0.
          CYCLE
       END IF

       fluxtrans = 0.

       IF ((lam(1) > lamfilt(i,1)) .OR. &
            (lam(nlam) < lamfilt(i,nlamfilt(i)))) THEN
          
          fluxtrans = -1.
          
       ELSE
          
          k = 1
          DO WHILE (lam(k+1) < lamfilt(i,1))
             k = k + 1
          END DO
          j = 1
          laminf = lamfilt(i,j)
          transinf = 0.
          fluxinf = 0.
          lamsup = laminf

          DO WHILE (j+1 <= nlamfilt(i) .AND. k+1 <= nlam)

             IF (lamfilt(i,j+1) < lam(k+1)) THEN
                lamsup = lamfilt(i,j+1)
                fluxsup = interploglog(lam(k), lam(k+1), flux(k), &
                     flux(k+1), lamsup)
                transsup = trans(i,j+1) * lamsup ** typetrans(i)         
                j = j + 1
             ELSE
                lamsup = lam(k+1)
                fluxsup = flux(k+1)
                transsup = interplinlin(lamfilt(i,j), lamfilt(i,j+1), &
                     trans(i,j), trans(i,j+1), lamsup) * &
                     lamsup ** typetrans(i)  
                k = k + 1
             END IF

             fluxtrans = fluxtrans + (transinf * fluxinf + &
                  transsup * fluxsup) * (lamsup - laminf) / 2
             laminf = lamsup
             fluxinf = fluxsup
             transinf = transsup

          END DO

          fluxfilt(i) = fluxtrans / area(i)

       END IF

    END DO
    
  END SUBROUTINE calcfluxfilt
  
  
  REAL(KIND=8) FUNCTION interplinlin(x1, x2, y1, y2, t)
    
    IMPLICIT NONE
    REAL(KIND=8) :: x1, x2, y1, y2, t
    
    interplinlin = y1 + (y2 - y1) * (t - x1) / (x2 - x1)
    
  END FUNCTION interplinlin

  
  REAL(KIND=8) FUNCTION interploglog(x1, x2, y1, y2, t)
    
    IMPLICIT NONE
    REAL(KIND=8) :: x1, x2, y1, y2, t
    REAL(KIND=8), PARAMETER :: eps = 1d-37
    
    IF (y1 > eps .AND. y2 > eps) THEN      
       interploglog = y1 * (y2 / y1) ** (LOG(t / x1) / LOG(x2 / x1))
    ELSE
       interploglog = 0.
    END IF

  END FUNCTION interploglog
      

  SUBROUTINE CorrIGM (lam, z, fluxcorr)

    ! IGM absorption according to Meiksin (2006)

    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: lam ! observed wavelength
    REAL(KIND=8), INTENT(IN) :: z ! redshift
    REAL(KIND=8), DIMENSION(:), INTENT(OUT) :: fluxcorr ! corr(lambda)

    INTEGER, PARAMETER :: nmax = 31, mmax = 10
    REAL(KIND=8), PARAMETER :: lamL = 912 ! Lyman limit in A
    REAL(KIND=8), DIMENSION(2:9), PARAMETER :: fac = &
         (/1., 0.348, 0.179, 0.109, 0.0722, 0.0508, 0.0373, 0.0283/)
    REAL(KIND=8), PARAMETER :: gamma = 0.2788 ! Gamma(0.5,1)

    INTEGER :: n, m
    REAL(KIND=8) :: taua, term1, term2
    REAL(KIND=8), DIMENSION(2:nmax) :: lamn
    REAL(KIND=8), DIMENSION(2:nmax,SIZE(lam)) :: zn, taun
    REAL(KIND=8), DIMENSION(SIZE(lam)) :: zL, tauL_IGM, term3, term4
    REAL(KIND=8), DIMENSION(SIZE(lam)) :: tauL_LLS, tau 

    DO n = 2, nmax
       lamn(n) = lamL / (1. - 1. / n ** 2)
       zn(n,:) = lam / lamn(n) - 1.
    END DO

    IF (z >= 4) THEN
       taua = 0.00058 * (1. + z) ** 4.5
    ELSE
       taua = 0.00211 * (1. + z) ** 3.7
    END IF

    IF (z >= 4) THEN
       taun(2,:) = 0.00058 * (1. + zn(2,:)) ** 4.5
    ELSE
       taun(2,:) = 0.00211 * (1. + zn(2,:)) ** 3.7
    END IF 
    DO n = 3, 5
       WHERE (zn(n,:) < 3)
          taun(n,:) = taua * fac(n) * (0.25 * (1. + zn(n,:))) ** (1. / 3.)
       ELSEWHERE
          taun(n,:) = taua * fac(n) * (0.25 * (1. + zn(n,:))) ** (1. / 6.)
       END WHERE
    END DO
    DO n = 6, 9 
       taun(n,:) = taua * fac(n) * (0.25 * (1. + zn(n,:))) ** (1. / 3.)
    END DO
    DO n = 10, nmax
       taun(n,:) = taun(9,:) * 720. / (n * (n ** 2 - 1.)) 
    END DO
    WHERE (zn >= z) taun = 0.

    zL = lam / lamL - 1.
    WHERE (zL >= z)
       tauL_IGM = 0.
    ELSEWHERE
       tauL_IGM = 0.805 * (1. + zL) ** 3 * (1. / (1. + zL) - 1. / (1. + z))
    END WHERE

    term1 = gamma - EXP(-1.)
    term2 = 0
    DO m = 0, mmax-1
       term2 = term2 + (-1.) ** m / (rfacul(m) * REAL((2 * m - 1), 8))
    END DO
    term3 = (1. + z) * (lam / lamL) ** 1.5 - (lam / lamL) ** 2.5
    term4 = 0.
    DO m = 1, mmax
       term4 = term4 + (2. * (-1.) ** m / &
            (rfacul(m) * REAL((2 * m - 1) * (6 * m - 5), 8))) * &
            ((1. + z) ** (2.5 - REAL(3 * m, 8)) * (lam / lamL) ** (3 * m) - &
            (lam / lamL) ** 2.5)
    END DO
    WHERE (zL >= z)
       tauL_LLS = 0.      
    ELSEWHERE
       tauL_LLS = 0.25 * ((term1 - term2) * term3 - term4) 
    END WHERE

    tau = SUM(taun, DIM=1) + tauL_IGM + tauL_LLS 
    fluxcorr = EXP(- tau)

  END SUBROUTINE CorrIGM


  REAL(KIND=8) FUNCTION rfacul(n)

    ! Computes real faculty from integer value

    INTEGER, INTENT(IN) :: n
    INTEGER :: imax, i

    IF (n == 0) THEN
       imax = 1
    ELSE
       imax = n 
    END IF

    rfacul = 1.
    DO i = 1, imax
      rfacul = rfacul * i 
    END DO

  END FUNCTION rfacul


  CHARACTER(LEN=8) FUNCTION IntToChar(intnum, nchar)
    
    ! Converts integer numbers (8 digits in maximum) to character strings 

    IMPLICIT NONE
    INTEGER, INTENT(IN) :: intnum, nchar
    REAL(KIND=8) :: rnum
    REAL(KIND=8), PARAMETER :: tol = 1d-12
    INTEGER :: absint, j, i, inum
    CHARACTER(LEN=nchar) :: cnum

    absint = ABS(intnum)
    rnum = (REAL(absint, 8) + tol) / 10. ** (nchar - 1)
 
    j = 0
    DO WHILE (rnum >= 10)
       rnum = rnum / 10
       j = j + 1
    END DO
    IF (j >= 1) PRINT*, '(AI1A)', 'IntToChar: Integer too long -> last ', j, &
         ' digits not considered'

    DO i = 1, nchar
       inum = INT(rnum) 
       cnum(i:i) = ACHAR(48 + inum)  
       rnum = ((rnum - REAL(inum, 8)) * 10.)
    END DO
     
    IntToChar = TRIM(cnum)
    
  END FUNCTION IntToChar


  REAL(KIND=8) FUNCTION LumDist(z, H0, Omega_M, Omega_L)

    ! Computes luminosity distance at redshift z in Mpc for given Lambda 
    ! cosmology (H_0 in (km/s)/Mpc, Omega_M, and Omega_Lambda) 
    ! Ref.: Correll et al. (1992) / Riess et al. (1998)

    IMPLICIT NONE
    REAL(KIND=8) :: z
    REAL(KIND=8), OPTIONAL :: H0, Omega_M, Omega_L
    REAL(KIND=8), PARAMETER :: H0_sc = 70, Omega_M_sc = .3, Omega_L_sc = .7
    REAL(KIND=8), PARAMETER :: tol = 1d-12
!    REAL(KIND=8), PARAMETER :: dz = 1d-3 ! accuracy of integration
    INTEGER, PARAMETER :: nstep = 1001 ! define the accuracy instead of dz
    REAL(KIND=8), PARAMETER :: c = 2.99792458d5 ! light speed in km/s
    INTEGER :: imax, i
    REAL(KIND=8) :: dl, dzi, zi, Omega_k

    IF (.NOT. PRESENT(H0)) H0 = H0_sc
    IF (.NOT. PRESENT(Omega_M)) Omega_M = Omega_M_sc
    IF (.NOT. PRESENT(Omega_L)) Omega_L = Omega_L_sc

    dl = 0
    IF (z > tol) THEN
!       imax = NINT(z / dz)
       imax = nstep
       dzi = z / imax
       DO i = 1, imax 
!          zi = (i - 0.5) * dz
          zi = (i - 0.5) * dzi
          dl = dl + dzi * ((1 + zi) ** 2 * (1 + Omega_M * zi) - &
               zi * (2 + zi) * Omega_L) ** (-0.5)
       END DO
    ELSE IF (z < -tol) THEN
      PRINT*, 'LumDist: z < 0 -> Assume z = 0!'
      z = 0 
    END IF

    Omega_k = 1 - Omega_M - Omega_L
    IF (ABS(Omega_k) > tol) THEN
       dl = dl * (ABS(Omega_k) ** 0.5)
       IF (Omega_k > 0) THEN
          dl = SINH(dl)
       ELSE IF (Omega_k < 0) THEN
          dl = SIN(dl)
       END IF
       dl = dl * (ABS(Omega_k) ** (-0.5))
    END IF

    LumDist = c * (H0 ** (-1)) * (1 + z) * dl


  END FUNCTION LumDist


end subroutine theoryfnu

END module compphotfnu
