module effdust_dh

! Author: Stefan Noll (2009) 

! Dust reddening of spectra calculated by '*crespec*' + consistent IR dust 
! emission

! Attenuation laws are computed assuming the sum of a Calzetti et al. (2000)
! law and a Drude profile with 'mean' wavelength [A], 'width' [A], and 
! 'ampl'(itude) (cf. Fitzpatrick & Massa 2007) times a reciprocal R_V-like
! value derived from the input parameter 'cr' (c_R) times a power law with a 
! slope 'delta'. The amount of attenuation is parametrised by 'av' (A_V) (see 
! subroutine 'attsed').

! The IR models of Dale & Helou (2002) parametrised by the slope alpha are 
! optionally taken to consider the dust-related energy transfer to the IR. 

! Optional consideration of nebular emission lines and interstellar absorption
! lines (see subroutine 'conslines')

! Optional consideration of a hot dust component due to a hidden AGN 
! (Siebenmorgen et al. models). Its relative strength 'frac_AGN' is given by 
! the ratio of its luminosity and the total dust luminosity. 


use ParamDef
contains



subroutine dust(Params,nirmod,irid,irslope,nirlam,irflux,irlam,nhdlam,&
     hdlam,hdflux,nlambdalin,lambdalin,emlin,abslin,nrange,lambdarange,&
     outflux, boutflux, coutflux,lambda,nlam,lam,fsed,fsedp,effpar,Lbol0)


IMPLICIT NONE

INTEGER, PARAMETER :: agemax = 67, lambdamax = 1221
!INPUT (AA)
CHARACTER(LEN=2), DIMENSION(:), ALLOCATABLE, INTENT(IN) :: irid
REAL(KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(IN) :: irslope, hdlam,lambdalin
REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE, INTENT(IN) :: irlam, irflux,& 
     hdflux,emlin, abslin, lambdarange
INTEGER, INTENT(IN) :: nirmod, nirlam, nhdlam, nlambdalin, nrange
! from mcrecsp
REAL(KIND=8), DIMENSION(lambdamax), INTENT(IN) :: outflux, boutflux, coutflux
REAL(KIND=8), DIMENSION(lambdamax),INTENT(IN) :: lambda
!END I (AA)

!OUTPUT (AA)
REAL(KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: lam, fsed, fsedp
REAL(KIND=8), INTENT(OUT) :: Lbol0
REAL(KIND=8), DIMENSION(6), INTENT(OUT) :: effpar
INTEGER, INTENT(OUT) :: nlam
!END O (AA)



CHARACTER(LEN=*), PARAMETER :: indir = 'inspec/' ! INPUT folder!
CHARACTER(LEN=*), PARAMETER :: prefix = 'e' ! OUTPUT!
INTEGER, PARAMETER :: ncol = 6 ! OUTPUT!
REAL(KIND=8), PARAMETER :: Lsol = 3.827d26 ! in W
REAL(KIND=8), PARAMETER :: Lmin = 1d-10 ! in Lsol
REAL(KIND=8), DIMENSION(2), PARAMETER :: limfuv = (/1400.,1600./) ! in A
REAL(KIND=8), DIMENSION(2), PARAMETER :: limv = (/5400.,5600./) ! in A
REAL(KIND=8), PARAMETER :: tol = 1d-12

CHARACTER(LEN=160) :: header, fmt, fmtlam, fmtflux
CHARACTER(LEN=80) :: filenames, infilelist, attspecname, attpar, outfilelist
CHARACTER(LEN=80) :: irmodpar, linpar, ircaltab, irmod, cdummy, interpoldat
CHARACTER(LEN=80) :: specin, outdir, spec, specout
CHARACTER(LEN=16) :: irlab
CHARACTER(LEN=1) :: flrelpar, sharp
INTEGER, DIMENSION(7) :: nv
INTEGER, DIMENSION(2) :: ntempl, fl_dust, fl_templ
INTEGER, DIMENSION(1) :: nir
INTEGER :: narg, ndot, h, n, nm, nw, na, nr, ns, ne, nf, nattmod
INTEGER :: nalpha, nagn, nfagn, nal, nat, nfa, m, i
INTEGER :: ndir, ncomp, nsppar, nlambda, ndustmod
INTEGER :: nc, ic, nsplam, irow, nrow, llim, ulim
!INTEGER :: status_open, status_read
REAL(KIND=8), DIMENSION(2,2) :: lim
REAL(KIND=8), DIMENSION(2) :: limrL
REAL(KIND=8) :: lamsep, rdummy, alpha0, fl_agn, fagn0, ffuv0, fv0
REAL(KIND=8) :: crx, Ldust0, ffuv, Afuv, fv, Avt, Ldust, Lbol


CHARACTER(LEN=32), DIMENSION(:), ALLOCATABLE :: agntempl
CHARACTER(LEN=16), DIMENSION(:), ALLOCATABLE :: emlinname, abslinname
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: mean, width, ampl, cr, delta
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: av, attfac
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: alpha
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: fagn
!Paoletto changed ratem and ratabs
REAL(KIND=8), DIMENSION(2) :: ratem, ratabs
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: lambdatest
REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: fluxc, fredc
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: flux, fred
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: fcont, corrfunc, flin
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: fsed0
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: fspir
 Type(ParamSet)  Params


! *** Get master file ***

narg = iargc()



! Seems ncomp always equal to 2 (3 components : outflux, boutflux, coutflux)
ncomp=2


do n=1,7
   nv(n)=1
end do
nm=1
nw=1
na=1
nr=1
ns=1
ne=1
nf=1
!READ (20, *) (nv(n), n = 1, 7)
ALLOCATE (mean(nv(1)))
mean(1)=2175.

!READ (20, *) (mean(nm), nm = 1, nv(1)) 
ALLOCATE (width(nv(2)))
width(1)=300.
!READ (20, *) (width(nw), nw = 1, nv(2))
ALLOCATE (ampl(nv(3)))
!READ (20, *) (ampl(na), na = 1, nv(3))
ampl(1)=Params%P(7)
ALLOCATE (cr(nv(4)))
cr(1)=Params%P(8)
!READ (20, *) (cr(nr), nr = 1, nv(4))
ALLOCATE (delta(nv(5)))
delta(1)=Params%P(9)
!READ (20, *) (delta(ns), ns = 1 , nv(5))
ALLOCATE (av(nv(6)))
av(1)=Params%P(10)
!READ (20, *) (av(ne), ne = 1, nv(6))
ALLOCATE (attfac(nv(7)))
attfac(1)=Params%P(11)
!READ (20, *) (attfac(nf), nf = 1, nv(7))
!READ (20, *) flrelpar
flrelpar="N"
!CLOSE (20)    



nattmod = nv(1) * nv(2) * nv(3) * nv(4) * nv(5) * nv(6) * nv(7)


! *** Read IR model parameters ***

IF (TRIM(irmodpar) /= '_') THEN


   irlab='irdh'
   lamsep=100500.
   ircaltab='dhcal'

   ircaltab = indir // TRIM(ircaltab) // '.dat'
   nalpha=1
   nagn=1
   nfagn=1
!   READ (22, *) nalpha, nagn, nfagn
   ALLOCATE (alpha(nalpha))
   alpha(1)=Params%P(12)

  

!   READ (22, *) (alpha(nal), nal = 1, nalpha)
   ALLOCATE (agntempl(nagn))
!   READ (22, *) (agntempl(nat), nat = 1, nagn)
   agntempl(1)='agnx_00125_1200_032'
   agntempl = indir // agntempl
   ALLOCATE (fagn(nfagn))
   fagn(1)= params%P(13)
!   READ (22, *) (fagn(nfa), nfa = 1, nfagn)
   

!   CLOSE (22)

ELSE

   nalpha = 1
   nagn = 1
   nfagn = 1

END IF


! *** Read IR model calibrations ***

ntempl(1)=2
ntempl(2)=2
lim(1,1)=3400
lim(1,2)=3600
lim(2,1)=1420
lim(2,2)=1490

   ratem(1)=1.
   ratem(2)=3.7

  ratabs(1)=1.
   ratabs(2)=3.7

limrl(1)=0.3
limrL(2)=0.1



   nlambda=lambdamax
   IF (TRIM(irmodpar) /= '_') THEN
      nlam = nirlam
   ELSE
      nlam = nlambda
   END IF

   ! Number of dust-related parameters and models
   IF (TRIM(irmodpar) /= '_') THEN   
      IF (MINVAL(fagn) < tol) THEN
         ndustmod = nattmod * nalpha * (1 + nagn * (nfagn - 1)) + 1
      ELSE
         ndustmod = nattmod * nalpha * nagn * nfagn + 1
      END IF
   ELSE
      ndustmod = nattmod + 1
   END IF
 


   IF (ncomp == 1) THEN
      nc = 0
   ELSE
      nc = ncomp
   END IF

!   ALLOCATE (lambda(nlambda))
   ALLOCATE (fluxc(0:nc,nlambda), fredc(0:nc,nlambda))
   ALLOCATE (flux(nlambda), fred(nlambda))
   ALLOCATE (fcont(nlambda), corrfunc(nlambda), flin(nlambda))

   ! Read unattenuated spectrum


   fluxc(0,:)=outflux(:)
   fluxc(1,:)=boutflux(:)
   fluxc(2,:)=coutflux(:)
   

!   READ (70, *) header
!   READ (70, *) (lambda(i), (fluxc(ic,i), ic = 0, nc), i = 1, nlambda)
   flux = fluxc(0,:)

   IF (.NOT. ALL(lambda == lambdalin)) PRINT*, &
        'Wavelengths of the model spectrum and the templates do not match!'  

   ! Bolometric luminosity related to stars
   Lbol0 = IntegrateHisto(lambda, flux) / Lsol

   ! Derive correction function for multiplication of absorption lines 
   CALL interpollines (lambda, flux, lambdarange, fcont)
   WHERE (flux == 0.)
      corrfunc = 0.
   ELSEWHERE
      corrfunc = fcont / flux
   ENDWHERE

   ! Consider spectral lines:
   fl_dust = (/0, 0/)

   


   CALL conslines (lambda, flux, ntempl, lim, ratem, emlin, ratabs, &
        abslin, corrfunc, fl_dust, fl_templ, flin)


   


   ! Compute f_FUV and f_V for unattenuated spectrum:
   ffuv0 = AverageRange(lambda, flin, limfuv(1), limfuv(2)) 
   fv0 = AverageRange(lambda, flin, limv(1), limv(2)) 

   ! Adapt set of model wavelengths, if IR models are considered
    
   IF (TRIM(irmodpar) /= '_') THEN
      ALLOCATE (lam(nirlam), fsed(nirlam), fsed0(nirlam), fsedp(nirlam))
      nlam = nirlam
      lam = irlam(1,:)
      nsplam = COUNT(lam < lamsep)
!      IF (.not. ALL(lam(1:nsplam) == lambda)) THEN
!         PRINT*, &
!           'Wavelengths of the stellar population and IR models do not match!',lambda(nsplam),&
!           '   ',lam(nsplam),' ',nsplam, lambda(nsplam)-lam(nsplam)
!         DO i=1,nsplam 
!            IF (lam(i) .NE. lambda(i)) PRINT*,'unmatched :',i,lam(i),lambda(i)
!            STOP
!         ENDDO
!      ENDIF
      ALLOCATE (fspir(nirlam-nsplam))
      CALL InterpolCurve (lambda(nsplam:), flux(nsplam:), lam(nsplam+1:), &
           fspir)
      fsed(nsplam+1:) = fspir
      fsed(:nsplam) = flin(:nsplam)
      alpha0 = 0
      DO nal = 1, nalpha
         nir = MINLOC(ABS(irslope - alpha(nal)))
         alpha0 = alpha0 + irslope(nir(1)) / nalpha
      END DO
      IF (nfagn == 1 .AND. fagn(1) < tol) THEN
         fl_agn = 0
      ELSE
         fl_agn = REAL((nagn - 1), 8) / 2. 
      END IF
      fagn0 = SUM(fagn) / nfagn
   ELSE 
      ALLOCATE (lam(nlambda), fsed(nlambda), fsed0(nlambda), fsedp(nlambda))
      nlam = nlambda
      lam = lambda
      fsed = flin
      alpha0 = 9.9999
      fl_agn = 0
      fagn0 = 9.999
   END IF

   ! Write wavelengths

   nrow = INT(nlam / ncol)
   IF (nrow * ncol < nlam) nrow = nrow + 1

!   WRITE (80, '(A,I1,A)') '# lambda[A] (', ncol, ' per row):'
!   DO irow = 1, nrow
!      llim = (irow - 1) * ncol + 1
!      ulim = llim + ncol - 1
!      IF (ulim > nlam) ulim = nlam
!      WRITE (80, fmtlam) (lam(i), i = llim, ulim)   
!   END DO

   ! Default value for c_R:
   ! Relation between c_R and delta for const. R_V?

   IF (flrelpar == 'Y' .OR. flrelpar == 'y') THEN
      crx = (nv(5) * SUM(cr) - nv(4) * SUM(delta)) / (nv(4) * nv(5)) 
   ELSE
      crx = SUM(cr) / nv(4) 
   END IF

   

   fsedp(:)=fsed(:)

nm=1
nw=1
na=1
nr=1
ns=1


IF (flrelpar == 'Y' .OR. flrelpar == 'y') THEN
   crx = cr(nr) - delta(ns) 
ELSE
   crx = cr(nr) 
END IF

ne=1
nf=1

!DO ne = 1, nv(6)
!   DO nf = 1, nv(7)

! Apply dust correction:
IF (ncomp == 1) THEN
   ! 1 SP component:
   CALL attsed (lambda, flux, mean(nm), width(nw), &
        ampl(na), crx, delta(ns), av(ne), fred)
ELSE
   ! >1 SP components:
   ! Comp. 1 is reddened by A_V; additional comp.
   ! are redddened by A_V * attfac.
   CALL attsed (lambda, fluxc(1,:), mean(nm), &
        width(nw), ampl(na), crx, delta(ns), &
        av(ne), fredc(1,:))
   fred = fredc(1,:)
   DO ic = 2, nc
      CALL attsed (lambda, fluxc(ic,:), mean(nm), &
           width(nw), ampl(na), crx, delta(ns), &
           av(ne) * attfac(nf), fredc(ic,:))          
      fred = fred + fredc(ic,:)
   END DO
END IF

                        ! Derive correction function for multiplication of 
                        ! absorption lines: 
CALL interpollines (lambda, fred, lambdarange, fcont)
WHERE (flux == 0.)
   corrfunc = 0.
ELSEWHERE
   corrfunc = fcont / fred
ENDWHERE

! Compute star-related Ldust:
Ldust0 = Lbol0 - IntegrateHisto(lambda, fred) / Lsol  
IF (Ldust0 < Lmin) Ldust0 = Lmin

! Dusty spectrum? -> Require interstellar lines:
WHERE (Ldust0/Lbol0 * (/1, 1/) >= limrL)
   fl_dust = 1
ELSEWHERE
   fl_dust = 0
ENDWHERE

! Consider spectral lines:
CALL conslines (lambda, fred, ntempl, lim, ratem, &
     emlin, ratabs, abslin, corrfunc, fl_dust, &
     fl_templ, flin)

! Compute A_FUV and A_V:
ffuv = AverageRange(lambda,flin,limfuv(1),limfuv(2))
Afuv = 2.5 * LOG10(ffuv0 / ffuv)
fv = AverageRange(lambda, flin, limv(1), limv(2))
Avt = 2.5 * LOG10(fv0 / fv)

! Consider IR models, if desired

nal=1
!DO nal = 1, nalpha
   
IF (TRIM(irmodpar) /= '_') THEN
   nir = MINLOC(ABS(irslope - alpha(nal)))
   fsed0 = irflux(nir(1),:) * Ldust0 * Lsol
   fsed0(nsplam+1:) = fsed0(nsplam+1:) + fspir
   fsed0(:nsplam) = fsed0(:nsplam) + flin(:nsplam)
   alpha0 = irslope(nir(1))
ELSE
   fsed0 = flin
   alpha0 = 9.9999
END IF
 
nfa=1  
!   DO nfa = 1, nfagn
nat=1
!      DO nat = 1, nagn
         
IF (TRIM(irmodpar) /= '_') THEN
   fagn0 = fagn(nfa)
   ! Consider AGN templates, if desired
   IF (fagn0 > tol) THEN
      ! Increase of Ldust and Lbol by AGN
      Ldust = Ldust0 / (1. - fagn0)
      Lbol = Lbol0 + Ldust - Ldust0
      fsed = fsed0 + hdflux(nat,:) * &
           (Ldust - Ldust0) * Lsol
      ! 1st AGN template has No. 0!
      fl_agn = nat - 1
   ELSE
      Ldust = Ldust0
      Lbol = Lbol0
      fsed = fsed0
      IF (nfagn == 1) THEN
         fl_agn = 0
      ELSE
         IF (nat == 1) THEN
            fl_agn = REAL((nagn - 1), 8) / 2.
         ELSE
!            CYCLE
         END IF
      END IF
   END IF
ELSE 
   Ldust = Ldust0
   Lbol = Lbol0
   fsed = fsed0
   fagn0 = 9.999
   fl_agn = 0
END IF
         
effpar(1)=Lbol
effpar(2)= Ldust
effpar(3)=Afuv
effpar(4)=Avt
effpar(5)=mean(nm)
effpar(6)=width(nw)


      
DEALLOCATE (fluxc, fredc, flux, fred, fcont, corrfunc, flin, &
      fsed0, fspir)




CONTAINS

  SUBROUTINE InterpolCurve (inx, infx, refx, outfx)

    ! Derive function values for a different set of data points as given for
    ! the function

    IMPLICIT NONE    
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: inx, infx, refx
    REAL(KIND=8), DIMENSION(:), INTENT(OUT) :: outfx
    INTEGER :: ninx, nrefx, j, i
    INTEGER, DIMENSION(1) :: inxpos
    REAL(KIND=8) :: minx, maxx

    ninx = SIZE(inx)
    minx = inx(1)
    maxx = inx(ninx)

    nrefx = SIZE(refx)
    IF (refx(1) > minx) THEN
       inxpos = MINLOC(inx, inx > refx(1))
       j = inxpos(1)
    ELSE
       j = 2
    END IF

    DO i = 1, nrefx 

       IF (refx(i) < minx .OR. refx(i) > maxx) THEN
          outfx(i) = 0.
       ELSE IF (refx(i) == maxx) THEN
          outfx(i) = infx(ninx)
       ELSE
          DO WHILE (refx(i) < inx(j-1) .OR. refx(i) >= inx(j))
             j = j + 1
          END DO
          outfx(i) = &
               interplinlin(inx(j-1), inx(j), infx(j-1), infx(j), refx(i))
       END IF

    END DO

  END SUBROUTINE InterpolCurve


  SUBROUTINE attsed &
       (lambda, flux, mean, width, ampl, cr, delta, av, fred, att)
    
    ! Applies a synthetic attenuation law: 
    ! ((Calzetti+00 + Lorentzian-like UV bump) / R_V-like par.) * power law
    ! For lambda < 1200A the Calzetti law is linearly extrapolated. 

    ! Parameters:
    ! UV bump: mean, width, ampl
    ! R_V-like par.: cr
    ! power law: delta
    ! attenuation: av

    ! For cr + delta = const. R_V is roughly constant. 
    ! -> Reason for introduction of factor 'ar'

    IMPLICIT NONE 
    REAL(KIND=8), INTENT(IN) :: lambda(:), flux(:)
    REAL(KIND=8), INTENT(IN) :: mean, width, ampl, cr, delta, av
    REAL(KIND=8), INTENT(INOUT) :: fred(:)
    REAL(KIND=8), INTENT(OUT), OPTIONAL :: att(:)
    REAL(KIND=8), PARAMETER :: rv0 = 4.05 ! Calzetti et al. (2000)
    REAL(KIND=8), PARAMETER :: ar = 1.12 
    REAL(KIND=8), PARAMETER :: maxtau = 100.
    REAL(KIND=8), DIMENSION(SIZE(lambda)) :: lam, clz, bump, fac, pow
    REAL(KIND=8), DIMENSION(SIZE(lambda)) :: tau, trans
    
    lam = lambda / 10000.
    WHERE (lam >= 0.63)
       clz = -1.857 + 1.040 / lam 
    ELSEWHERE (lam < 0.12)
       clz = -34.80 * (lam - 0.12) + 3.035
    ELSEWHERE
       clz = -2.156 + 1.509 / lam - 0.198 / (lam ** 2) + 0.011 / (lam ** 3)
    ENDWHERE
    clz = 2.659 * clz

    bump = ampl * (lambda * width) ** 2 / &
         ((lambda ** 2 - mean ** 2) ** 2 + (lambda * MAX(width, 1.)) ** 2)

    fac = (1 - ar * cr) / rv0 

    pow = (lambda / 5500.) ** delta

    tau = av * ((clz + bump) * fac + 1) * pow
    WHERE (tau < 0.) tau = 0.
    WHERE (tau > maxtau) tau = maxtau

    trans = 10 ** (-0.4 * tau)
    fred = flux * trans
    IF (PRESENT(att)) att = -2.5 * LOG10(fred / flux)
    
  END SUBROUTINE attsed
  

  SUBROUTINE interpollines (lambda, influx, lambdarange, outflux)

    ! Linear interpolation in wavelength ranges given by a list

    IMPLICIT NONE 
    REAL(KIND=8), INTENT(IN) :: lambda(:), influx(:)
    REAL(KIND=8), INTENT(IN) :: lambdarange(:,:)
    REAL(KIND=8), INTENT(INOUT) :: outflux(:)
    INTEGER :: nrange, nlambda, j, jmin, jmax, inline, i, imin, imax
    REAL(KIND=8), DIMENSION(2,SIZE(lambdarange,2)) :: rangeflux
    REAL(KIND=8) :: m

    outflux = influx

    nrange = SIZE(lambdarange, DIM=2)
    nlambda = SIZE(lambda)

    IF (nrange /= 0) THEN

       DO j = 1, nrange
          IF (lambdarange(1,j) > lambda(1)) THEN
             jmin = j
             IF (jmin > 1) PRINT '(A,1X,I1,1X,A)', 'interpollines:', &
                  jmin - 1, 'ranges below limit of spectrum!' 
             EXIT
          END IF
       END DO

       DO j = nrange, 1, -1
          IF (lambdarange(2,j) < lambda(nlambda)) THEN
             jmax = j
             IF (jmax < nrange) PRINT '(A,1X,I1,1X,A)', 'interpollines:', &
                  nrange - jmax, 'ranges above limit of spectrum!' 
             EXIT
          END IF
       END DO

       j = jmin
       inline = 0
       
       DO i = 2, nlambda

          IF (inline == 0 .AND. lambda(i) >= lambdarange(1,j)) THEN
             imin = i
             rangeflux(1,j) = &
                  ((influx(i) - influx(i-1)) / (lambda(i) - lambda(i-1))) * &
                  (lambdarange(1,j) - lambda(i-1)) + influx(i-1)   
             inline = 1
          END IF

          IF (inline == 1 .AND. lambda(i) >= lambdarange(2,j)) THEN
             imax = i-1
             rangeflux(2,j) = &
                  ((influx(i) - influx(i-1)) / (lambda(i) - lambda(i-1))) * &
                  (lambdarange(2,j) - lambda(i-1)) + influx(i-1)   
             m = (rangeflux(2,j) - rangeflux(1,j)) / &
                  (lambdarange(2,j) - lambdarange(1,j))
             outflux(imin:imax) = &
                  m * (lambda(imin:imax) - lambdarange(1,j)) + rangeflux(1,j) 
             j = j + 1
             IF (j > jmax) EXIT
             IF (lambdarange(1,j) == lambdarange(2,j-1)) THEN
                rangeflux(1,j) = rangeflux(2,j-1)
                imin = imax + 1
                inline = 1
             ELSE
                inline = 0
             END IF
          END IF

       END DO

    END IF

  END SUBROUTINE interpollines


  SUBROUTINE conslines (lambda, influx, ntempl, lim, ratem, emlin, ratabs, &
       abslin, corrfunc, fl_dust, fl_templ, outflux)

    ! Consider optical (nebular emission) and UV (interstellar absorption) 
    ! lines

    ! Use phenomenological approach:
    ! Optical lines (additive)
    ! -> based on Kinney et al. (1996) spectra of nearby starbursts
    ! UV lines (multiplicative)
    ! -> based on Noll et al. (2004) spectra of 1 < z < 4 spectra in the FDF
 
    ! Subroutine needs the function 'AverageRange'

    IMPLICIT NONE 
    INTEGER, DIMENSION(2), INTENT(IN) :: ntempl
    INTEGER, DIMENSION(2), INTENT(IN) :: fl_dust
    REAL(KIND=8), INTENT(IN) :: lambda(:), influx(:), corrfunc(:) ! nlambda
    REAL(KIND=8), INTENT(IN) :: ratem(:), ratabs(:) ! nt1/nt2
    REAL(KIND=8), INTENT(IN) :: emlin(:,:), abslin(:,:) ! nt1/nt2,nlambda
    REAL(KIND=8), DIMENSION(2,2), INTENT(IN) :: lim
    INTEGER, DIMENSION(2), INTENT(OUT) :: fl_templ
    REAL(KIND=8), INTENT(INOUT) :: outflux(:)
    INTEGER :: it, emlintype, abslintype
    REAL(KIND=8) :: mean1, mean2, rat21

    outflux = influx
      
    IF (ntempl(1) /= 0 .OR. ntempl(2) /= 0) THEN
       mean1 = AverageRange(lambda, influx, lim(1,1), lim(1,2))
       mean2 = AverageRange(lambda, influx, lim(2,1), lim(2,2))
       rat21 = mean2 / mean1
    END IF
       
    emlintype = 0
    
    IF (ntempl(1) /= 0) THEN
       DO it = 1, ntempl(1)
          IF (rat21 < ratem(it)) THEN
             IF (it > 1 .OR. fl_dust(1) == 0) EXIT 
          END IF
          emlintype = it
         
       END DO
    END IF
    IF (emlintype > 0) & 
         outflux = outflux + mean1 * emlin(emlintype,:)
 
    abslintype = 0
    IF (ntempl(2) /= 0) THEN
       DO it = 1, ntempl(2)
          IF (rat21 < ratabs(it)) THEN
             IF (it > 1 .OR. fl_dust(2) == 0) EXIT 
          END IF
          abslintype = it 
       END DO
    END IF
    IF (abslintype > 0) &
         outflux = outflux * (1. + corrfunc * abslin(abslintype,:))

    fl_templ = (/emlintype, abslintype/)

    


  END SUBROUTINE conslines


  REAL(KIND=8) FUNCTION IntegrateHisto(lambda, flux)

    ! Integrates histogram

    IMPLICIT NONE
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: lambda, flux
    INTEGER :: imax, i
    REAL(KIND=8) :: dlambda

    imax = MIN(SIZE(lambda), SIZE(flux))
    IF (imax < SIZE(lambda)) PRINT*, &
         'IntegrateHisto: More wavelengths than fluxes!'
    IF (imax > SIZE(lambda)) PRINT*, & 
         'IntegrateHisto: Less wavelengths than fluxes!' 

    IntegrateHisto = 0

    DO i = 1, imax

       IF (i == 1) THEN
          dlambda = lambda(i+1) - lambda(i)
       ELSE IF (i == imax) THEN
          dlambda = lambda(i) - lambda(i-1)
       ELSE
          dlambda = (lambda(i+1) - lambda(i-1)) / 2
       END IF

       IntegrateHisto = IntegrateHisto + dlambda * flux(i)

    END DO

  END FUNCTION IntegrateHisto


  REAL(KIND=8) FUNCTION interplinlin(x1, x2, y1, y2, t)
    
    IMPLICIT NONE
    REAL(KIND=8) :: x1, x2, y1, y2, t
    
    interplinlin = y1 + (y2 - y1) * (t - x1) / (x2 - x1)
    
  END FUNCTION interplinlin


  REAL(KIND=8) FUNCTION AverageRange(lambda, flux, llim, ulim)

    ! Averages curve in a given range

    IMPLICIT NONE
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: lambda, flux
    REAL(KIND=8), INTENT(IN) :: llim, ulim
    INTEGER :: imax, i
    REAL(KIND=8) :: llam, ulam, dlambda

    imax = MIN(SIZE(lambda), SIZE(flux))
    IF (imax < SIZE(lambda)) PRINT*, &
         'AverageRange: More wavelengths than fluxes!'
    IF (imax > SIZE(lambda)) PRINT*, & 
         'AverageRange: Less wavelengths than fluxes!' 

    AverageRange = 0.

    IF (ulim <= llim) THEN

       PRINT*, 'AverageRange: Upper limit <= lower limit!'
     
    ELSE

       DO i = 1, imax

          IF (i == 1) THEN
             llam = lambda(i) - (lambda(i+1) - lambda(i)) / 2
          ELSE
             llam = (lambda(i) + lambda(i-1)) / 2
          END IF

          IF (i == imax) THEN
             ulam = lambda(i) + (lambda(i) - lambda(i-1)) / 2
          ELSE
             ulam = (lambda(i+1) + lambda(i)) / 2
          END IF
             
          IF (llim <= ulam .AND. ulim >= llam) THEN

             IF (llim > llam .OR. ulim < ulam) THEN
                IF (llim > llam) dlambda = MAX(ulam - llim, 0.)
                IF (ulim < ulam) dlambda = MAX(ulim - llam, 0.)
             ELSE                
                dlambda = ulam - llam
             END IF

             AverageRange = AverageRange + dlambda * flux(i)

          END IF

       END DO

       AverageRange = AverageRange / (ulim - llim)

    END IF

  END FUNCTION AverageRange

end subroutine dust

END module effdust_dh


