module mcrecsp
! Author: Stefan Noll (2009) 

! Creates single spectrum files from combined young + old tau or box models 
! based on Maraston (2005) SSP models

use cmbtypes
use settings
use ParamDef



contains
subroutine stellpop(Params,age,mstar,flux,dage,d4000,lambda,nage,iagemax,outflux,boutflux,coutflux,cresppar,ratage,ratagetau)

IMPLICIT NONE

CHARACTER(LEN=*), PARAMETER :: indir = 'inspec/' ! INPUT folder!
INTEGER, PARAMETER :: agemax = 67, lambdamax = 1221
REAL(KIND=8), PARAMETER :: sfrmin = 1d-18 ! in yr^-1
REAL(KIND=8), PARAMETER :: tol = 1d-12
REAL(KIND=8), PARAMETER :: c = 2.99792458d18 ! light speed in A/s
REAL(KIND=8), DIMENSION(2), PARAMETER :: bwl = (/3850., 3950./)
REAL(KIND=8), DIMENSION(2), PARAMETER :: rwl = (/4000., 4100./)
REAL(KIND=8), DIMENSION(4), INTENT(OUT) :: cresppar
!INPUT AND OUTPUT (AA):

REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE, INTENT(IN) :: age, mstar
REAL(KIND=8), DIMENSION(:,:,:), ALLOCATABLE,INTENT(IN) :: flux
REAL(KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(IN) :: dage, d4000
REAL(KIND=8), DIMENSION(lambdamax),INTENT(IN) :: lambda
INTEGER, INTENT(IN) :: nage
INTEGER, DIMENSION(:), ALLOCATABLE, INTENT(IN) :: iagemax
REAL(KIND=8), DIMENSION(lambdamax), INTENT(OUT) :: outflux, boutflux, coutflux
REAL(KIND=8) ratage,ratagetau

!END I&O (AA)

CHARACTER(LEN=160) :: fmt,myfile1
CHARACTER(LEN=80) :: header, specpar, filelist, bayespar
CHARACTER(LEN=80) :: idin, specin, massin, d4000dat, specout
CHARACTER(LEN=8) :: metc, tauc, ctauc, cagec, btauc, bagec
CHARACTER(LEN=8) :: btaucc, bagecc, bfracc
CHARACTER(LEN=1) :: imf, csfmod, bsfmod
INTEGER, DIMENSION(6) :: nv
INTEGER, DIMENSION(2) :: fl_dex, bfac
INTEGER, DIMENSION(1) :: loc
INTEGER :: h, n, nm, nt, nc, nu, nb, nf, fl_burst
INTEGER :: i, j, fl_par, hmax, iage, icage, ibage
INTEGER :: status_read, status_open
REAL(KIND=8) :: rdummy, mindex, csfr, cmwage,pippo
REAL(KIND=8) :: bsfr, bmwage, fmstar, sfr, mwage, btaur, bager, bfracr
REAL(KIND=8) :: bw, rw
REAL(KIND=8), DIMENSION(1) :: d4000x,dagex
REAL(KIND=8), DIMENSION(lambdamax) :: fnu
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: met, ctau, cage, btau, bage, bfrac 
REAL(KIND=8), DIMENSION(:,:,:,:), ALLOCATABLE :: cflux, bflux
REAL(KIND=8), DIMENSION(:,:,:), ALLOCATABLE :: cmstar, bmstar
 Type(ParamSet)  Params


do n=1,6 
nv(n)=1
enddo

nm=1
nt=1
nc=1
nu=1
nb=1
nf=1
ALLOCATE (met(nv(1)), ctau(nv(2)), cage(nv(3)))
ALLOCATE (btau(nv(4)), bage(nv(5)), bfrac(nv(6)))

met(1)=Params%P(1)
ctau(1)=Params%P(2)
cage(1)=Params%P(3)
btau(1)=Params%P(4)
bage(1)=Params%P(5)
bfrac(1)=Params%P(6)



imf='s'
csfmod='t'
bsfmod= 't'


IF (bage(nv(1)) > cage(nv(1))) PRINT*, 'Burst older than oldest CSP model!'
!READ (20, *) (bfrac(nf), nf = 1, nv(6))
!READ (20, '(A1)') imf
IF (imf /= 's' .AND. imf /= 'k') THEN
   PRINT*, 'Wrong IMF identifier! -> Assume Salpeter IMF!'
   imf = 's'
END IF
!READ (20, *) csfmod, bsfmod
IF (csfmod /= 't' .AND. csfmod /= 'b') THEN
   PRINT*, 'Wrong SFH identifier for old CSP model! -> Assume tau model!'
   csfmod = 't'
END IF
IF (bsfmod /= 't' .AND. bsfmod /= 'b') THEN
   PRINT*, 'Wrong SFH identifier for burst! -> Assume tau model!'
   bsfmod = 't'
END IF



!ratage= 5  !1.4 !3.5
!ratagetau= 0.1 ! 20 ! 0.25 !0.1

!READ (20, *) ratage
!READ (20, *) ratagetau

!CLOSE (20)    

fl_burst = 1
! No burst:
IF (bfrac(1) < tol .AND. nv(6) == 1) THEN
 PRINT*, 'No bursts considered!'
 btau = MAXVAL(ctau)
 bage = MINVAL(cage)
 ratage = 0.
 fl_burst = 0
END IF



! *** Calculate CSP models *** 
    
ALLOCATE (cflux(nv(1),nv(2),nv(3),lambdamax))
ALLOCATE (cmstar(nv(1),nv(2),nv(3)))
ALLOCATE (bflux(nv(1),nv(4),nv(5),lambdamax))
ALLOCATE (bmstar(nv(1),nv(4),nv(5)))

DO nm = 1, nv(1)

   DO nt = 1, nv(2)
      
      IF (csfmod == 'b') THEN
         CALL CalcBoxMod (ctau(nt), cage, age(nm,:iagemax(nm)), &
              flux(nm,:iagemax(nm),:), mstar(nm,:iagemax(nm)), &
              cflux(nm,nt,:,:), cmstar(nm,nt,:)) 
      ELSE 
         CALL CalctauMod (ctau(nt), cage, age(nm,:iagemax(nm)), &
              flux(nm,:iagemax(nm),:), mstar(nm,:iagemax(nm)), &
              cflux(nm,nt,:,:), cmstar(nm,nt,:)) 

       

      END IF
   END DO

   IF (fl_burst == 1) THEN
      DO nu = 1, nv(4)
         IF (bsfmod == 'b') THEN
            CALL CalcBoxMod (btau(nu), bage, age(nm,:iagemax(nm)), &
                 flux(nm,:iagemax(nm),:), mstar(nm,:iagemax(nm)), &
                 bflux(nm,nu,:,:), bmstar(nm,nu,:))

            

         ELSE
            CALL CalctauMod (btau(nu), bage, age(nm,:iagemax(nm)), &
                 flux(nm,:iagemax(nm),:), mstar(nm,:iagemax(nm)), &
                 bflux(nm,nu,:,:), bmstar(nm,nu,:))
         
         END IF
      END DO
   END IF

END DO


! *** Average burst parameters in dex? ***

! The selected tau values and ages of the burst models are averaged, if the 
! burst fraction is zero. In order to provide correct mean values, the 
! averaging has to be carried out in dex, if the dynamical range of the 
! parameter is larger than the limit given in the parameter file for the 
! Bayesian analysis (last entry).   



mindex=1.0

fl_dex = 0
IF (LOG10(MAXVAL(btau) / MINVAL(btau)) > mindex - tol) fl_dex(1) = 1
IF (LOG10(MAXVAL(bage) / MINVAL(bage)) > mindex - tol) fl_dex(2) = 1



! *** Calculate combined CSP models and write output spectra ***


DO nm = 1, nv(1)

! mods from AA
!nm=1
!nt=1
!icage=1
!nu=1
!ibage=1
!nf=1

!metc = IntToChar(NINT(met(nm) * 1000), 2)

   DO nt = 1, nv(2)

ctauc = IntToChar(NINT(ctau(nt) * 10000), 6)

      DO icage = 1, nv(3)

cagec = IntToChar(NINT(cage(icage) * 10000), 6)

! SFR [Msol/yr] per solar mass for old CSP models:
IF (csfmod == 'b') THEN
   IF (ctau(nt) >= cage(icage)) THEN
      csfr = 1d-9 / cage(icage)
   ELSE
      csfr = 0.
   END IF
ELSE
   csfr = 1d-9 / (ctau(nt) * (EXP(cage(icage) / ctau(nt)) - 1))
END IF

! Mass-weighted age [Gyr] for old CSP models:
IF (csfmod == 'b') THEN
   cmwage = (cage(icage) + MAX(cage(icage) - ctau(nt), 0.)) / 2
ELSE
   cmwage = cage(icage) / (1 - EXP(-cage(icage) / ctau(nt))) &
        - ctau(nt)
END IF

         DO nu = 1, nv(4)

IF (fl_burst == 1) btauc = IntToChar(NINT(btau(nu) * 10000), 6)

            DO ibage = 1, nv(5)

bfac(1) = 1 

IF (fl_burst == 1) THEN
   
   bagec = IntToChar(NINT(bage(ibage) * 10000), 6)
   
   ! SFR [Msol/yr] per solar mass for young CSP models 
   ! (bursts):
   IF (bsfmod == 'b') THEN
      IF (btau(nu) >= bage(ibage)) THEN
         bsfr = 1d-9 / bage(ibage)
      ELSE
         bsfr = 0.
      END IF
   ELSE
      bsfr = 1d-9 / &
           (btau(nu) * (EXP(bage(ibage) / btau(nu)) - 1))
   END IF
   
   ! Mass-weighted age [Gyr] for young CSP models:
   IF (bsfmod == 'b') THEN
      bmwage = &
           (bage(ibage) + MAX(bage(ibage) - btau(nu), 0.)) / 2
   ELSE
      bmwage = bage(ibage) / &
           (1 - EXP(-bage(ibage) / btau(nu))) - btau(nu)
   END IF
   
ELSE
                  
   bfac(1) = 0

END IF

               DO nf = 1, nv(6)


bfac(2) = 1
IF (bfrac(nf) < -tol .OR. bfrac(nf) > 1 - tol) THEN
   PRINT*, 'Invalid burst fraction! -> Skip!'
   CYCLE
END IF
IF (bfrac(nf) < tol) bfac(2) = 0
IF (bfac(2) == 0 .AND. (nu > 1 .OR. ibage > 1)) CYCLE

IF (bfac(1) == 0 .OR. bfac(2) == 0) THEN
   
   IF (nv(6) == 1) THEN
      ! No burst:
      IF (cage(icage) / ctau(nt) - ratagetau > tol) CYCLE
      btaur = 0.
      bager = 0.
   ELSE
      IF (fl_dex(1) == 1) THEN
         btaur = 10 ** (SUM(LOG10(btau)) / nv(4))
      ELSE
         btaur = SUM(btau) / nv(4)
      END IF
      IF (fl_dex(2) == 1) THEN
         bager = 10 ** (SUM(LOG10(bage)) / nv(5))
      ELSE
         bager = SUM(bage) / nv(5)
      END IF
   END IF
   
   boutflux = 0.
   coutflux = cflux(nm,nt,icage,:)
   fmstar = cmstar(nm,nt,icage)
   sfr = csfr
   mwage = cmwage
   btaucc = '00000'
   bagecc = '0000'
   bfracr = 0.
   bfracc = '0000'
   
ELSE
   
   

   IF (cage(icage) / bage(ibage) - ratage < -tol) CYCLE
   IF (bage(ibage) / btau(nu) - ratagetau > tol) CYCLE
   boutflux = bfrac(nf) * bflux(nm,nu,ibage,:)
   coutflux = (1 - bfrac(nf)) * cflux(nm,nt,icage,:)
   
   



   fmstar = (1 - bfrac(nf)) * cmstar(nm,nt,icage) &
        + bfrac(nf) * bmstar(nm,nu,ibage)
   sfr = (1 - bfrac(nf)) * csfr + bfrac(nf) * bsfr
   mwage = (1 - bfrac(nf)) * cmwage + bfrac(nf) * bmwage
   btaur = btau(nu)
   btaucc = btauc
   bager = bage(ibage)
   bagecc = bagec
   bfracr = bfrac(nf)
   bfracc = IntToChar(NINT(bfrac(nf) * 10000), 4)
   
END IF

! Avoid SFR = 0:
IF (sfr < sfrmin) sfr = sfrmin
! erg/s -> W 
boutflux = boutflux * 1d-7
coutflux = coutflux * 1d-7


! Total flux of stellar populations: 
outflux = boutflux + coutflux  

! D4000-related age [Gyr] for old tau models:
fnu = (lambda ** 2 / c) * outflux ! Flux in W/Hz
bw = AverageRange(lambda, fnu, bwl(1), bwl(2))
rw = AverageRange(lambda, fnu, rwl(1), rwl(2))
d4000x(1) = rw / bw 
CALL InterpolCurve (d4000, dage, d4000x, dagex)


cresppar(1)=fmstar
cresppar(2)=sfr
cresppar(3)=mwage
cresppar(4)=dagex(1)




IF (MAXVAL(outflux) < tol) THEN
   IF (MAXVAL(bflux(nm,nu,ibage,:)) < tol) &
        PRINT*, 'No data in young SP model!'
   IF (MAXVAL(cflux(nm,nt,icage,:)) < tol) &
        PRINT*, 'No data in old SP model!'
END IF

!                  CLOSE (80)

END DO

            END DO

         END DO

      END DO

   END DO
   
END DO


!CLOSE (70)


CONTAINS

  SUBROUTINE CalcBoxMod (tau, ages, iages, iflux, imstar, oflux, omstar)

    ! Derives box models with constant SF over a limited period tau from SSP 
    ! models. The corresponding fluxes and M_star fractions are computed.

    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN) :: tau
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: ages
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: iages
    REAL(KIND=8), DIMENSION(:,:), INTENT(IN) :: iflux
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: imstar
    REAL(KIND=8), DIMENSION(:,:), INTENT(OUT) :: oflux
    REAL(KIND=8), DIMENSION(:), INTENT(OUT) :: omstar

    INTEGER :: imax, i, jmax, j
    REAL(KIND=8), DIMENSION(SIZE(iages)) :: w
    REAL(KIND=8), DIMENSION(0:SIZE(iages)) :: lim
    REAL(KIND=8), DIMENSION(SIZE(iflux, DIM=2)) :: fx
    REAL(KIND=8) :: tend, ms, ws, ww, wsum

    ! Limits of age bins

    imax = SIZE(iages)

    DO i = 1, imax
     
       IF (i == 1) lim(0) = 0.
       IF (i == imax) THEN
          lim(i) = iages(imax)
       ELSE
          lim(i) = (iages(i) + iages(i+1)) / 2
       !   lim(i) = 10 ** ((LOG10(iages(i)) + LOG10(iages(i+1))) / 2)
       ENDIF

    END DO

    ! Fluxes and M_star fractions for given tau and ages 

    jmax = SIZE(ages)

    DO j = 1, jmax

       tend = ages(j) - tau
       fx = 0.
       ms = 0. 
       ws = 0.
     
       DO i = 1, imax

          IF (lim(i) > ages(j)) EXIT

          ! Weights of different SSP ages 
          IF (lim(i-1) >= tend) THEN
             w(i) = lim(i) - lim(i-1) 
          ELSE IF (lim(i-1) < tend .AND. lim(i) >= tend) THEN
             w(i) = lim(i) - tend 
          ELSE IF (lim(i) < tend) THEN
             w(i) = 0.
          END IF

          fx = fx + w(i) * iflux(i,:)
          ms = ms + w(i) * imstar(i)
          ws = ws + w(i)

       END DO

       ! Weight of last (incomplete) bin:
       IF (lim(i-1) < tend) THEN
          ww = ages(j) - tend  
       ELSE
          ww = ages(j) - lim(i-1) 
       END IF

       wsum = ws + ww
       oflux(j,:) = (fx + ww * iflux(i,:)) / wsum
       omstar(j) = (ms + ww * imstar(i)) / wsum

     


    END DO

  END SUBROUTINE CalcBoxMod


  SUBROUTINE CalctauMod (tau, ages, iages, iflux, imstar, oflux, omstar)

    ! Derives tau models with exponentially-decaying SFR from SSP models.
    ! The corresponding fluxes and M_star fractions are computed. 

    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN) :: tau
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: ages
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: iages
    REAL(KIND=8), DIMENSION(:,:), INTENT(IN) :: iflux
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: imstar
    REAL(KIND=8), DIMENSION(:,:), INTENT(OUT) :: oflux
    REAL(KIND=8), DIMENSION(:), INTENT(OUT) :: omstar

    INTEGER :: imax, i, jmax, j, imin
    REAL(KIND=8), DIMENSION(SIZE(iages)) :: w
    REAL(KIND=8), DIMENSION(0:SIZE(iages)) :: lim
    REAL(KIND=8), DIMENSION(SIZE(iflux, DIM=2)) :: fx
    REAL(KIND=8) :: ms, ws, ww, wsum

    ! Weights of different SSP ages 
    ! from integrated e-function for each age bin

    imax = SIZE(iages)

    DO i = 1, imax
      
       IF (i == 1) lim(0) = 0.
       IF (i == imax) THEN
          lim(i) = iages(imax)
       ELSE
          lim(i) = (iages(i) + iages(i+1)) / 2
       !   lim(i) = 10 ** ((LOG10(iages(i)) + LOG10(iages(i+1))) / 2)
       ENDIF

       w(i) = EXP(lim(i) / tau) - EXP(lim(i-1) / tau)     
       !    * tau * EXP(-ages(j) / tau) / wsum
     
    END DO

    ! Fluxes and M_star fractions for given tau and ages 
    ! Reduction of computing time for ascending order of ages

    jmax = SIZE(ages)

    DO j = 1, jmax

       IF (j == 1 .OR. ages(j) < ages(j-1)) THEN
          imin = 1
          fx = 0.
          ms = 0. 
          ws = 0.
       END IF
     
       DO i = imin, imax
          IF (lim(i) > ages(j)) EXIT
          fx = fx + w(i) * iflux(i,:)
          ms = ms + w(i) * imstar(i)
          ws = ws + w(i)
       END DO

       ww = EXP(ages(j) / tau) - EXP(lim(i-1) / tau)
       wsum = ws + ww
       oflux(j,:) = (fx + ww * iflux(i,:)) / wsum
       omstar(j) = (ms + ww * imstar(i)) / wsum
      

     
      


       imin = i

    END DO

  END SUBROUTINE CalctauMod


  SUBROUTINE InterpolCurve (inx, infx, refx, outfx)

    ! Derives function values for a different set of data points as given for
    ! the function

    IMPLICIT NONE    
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: inx, infx, refx
    REAL(KIND=8), DIMENSION(:), INTENT(OUT) :: outfx
    INTEGER :: ninx, nrefx, j, i
    INTEGER, DIMENSION(1) :: inxpos
    REAL(KIND=8) :: minx, maxx

    ninx = SIZE(inx)
    minx = inx(1)
    maxx = inx(ninx)

    nrefx = SIZE(refx)
    IF (refx(1) > minx) THEN
       inxpos = MINLOC(inx, inx > refx(1))
       j = inxpos(1)
    ELSE
       j = 2
    END IF

    DO i = 1, nrefx 

       IF (refx(i) < minx .OR. refx(i) > maxx) THEN
          outfx(i) = 0.
       ELSE IF (refx(i) == maxx) THEN
          outfx(i) = infx(ninx)
       ELSE
          DO WHILE (refx(i) < inx(j-1) .OR. refx(i) >= inx(j))
             j = j + 1
          END DO
          outfx(i) = &
               interplinlin(inx(j-1), inx(j), infx(j-1), infx(j), refx(i))
       END IF

    END DO

  END SUBROUTINE InterpolCurve


  CHARACTER(LEN=8) FUNCTION IntToChar(intnum, nchar)
    
    ! Converts integer numbers (8 digits in maximum) to character strings 

    IMPLICIT NONE
    INTEGER, INTENT(IN) :: intnum, nchar
    REAL(KIND=8) :: rnum
    REAL(KIND=8), PARAMETER :: tol = 1d-12
    INTEGER :: absint, j, i, inum
    CHARACTER(LEN=nchar) :: cnum

    absint = ABS(intnum)
    rnum = (REAL(absint, 8) + tol) / 10. ** (nchar - 1)
 
    j = 0
    DO WHILE (rnum >= 10)
       rnum = rnum / 10
       j = j + 1
    END DO
    IF (j >= 1) PRINT*, '(AI1A)', 'IntToChar: Integer too long -> last ', j, &
         ' digits not considered'

    DO i = 1, nchar
       inum = INT(rnum) 
       cnum(i:i) = ACHAR(48 + inum)  
       rnum = ((rnum - REAL(inum, 8)) * 10.)
    END DO
     
    IntToChar = TRIM(cnum)
    
  END FUNCTION IntToChar


  REAL(KIND=8) FUNCTION IntegrateRange(lambda, flux, llim, ulim)

    ! Integrates curve in a given range

    IMPLICIT NONE
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: lambda, flux
    REAL(KIND=8), INTENT(IN) :: llim, ulim
    INTEGER :: imax, i
    REAL(KIND=8) :: llam, ulam, dlambda

    imax = MIN(SIZE(lambda), SIZE(flux))
    IF (imax < SIZE(lambda)) PRINT*, &
         'IntegrateRange: More wavelengths than fluxes!'
    IF (imax > SIZE(lambda)) PRINT*, & 
         'IntegrateRange: Less wavelengths than fluxes!' 

    IntegrateRange = 0.

    IF (ulim <= llim) THEN

       PRINT*, 'IntegrateRange: Upper limit <= lower limit!'
     
    ELSE

       DO i = 1, imax

          IF (i == 1) THEN
             llam = lambda(i) - (lambda(i+1) - lambda(i)) / 2
          ELSE
             llam = (lambda(i) + lambda(i-1)) / 2
          END IF

          IF (i == imax) THEN
             ulam = lambda(i) + (lambda(i) - lambda(i-1)) / 2
          ELSE
             ulam = (lambda(i+1) + lambda(i)) / 2
          END IF
             
          IF (llim <= ulam .AND. ulim >= llam) THEN

             IF (llim > llam .OR. ulim < ulam) THEN
                IF (llim > llam) dlambda = MAX(ulam - llim, 0.)
                IF (ulim < ulam) dlambda = MAX(ulim - llam, 0.)
             ELSE                
                dlambda = ulam - llam
             END IF

             IntegrateRange = IntegrateRange + dlambda * flux(i)

          END IF

       END DO

    END IF

  END FUNCTION IntegrateRange


  REAL(KIND=8) FUNCTION AverageRange(lambda, flux, llim, ulim)

    ! Averages curve in a given range
    ! Needs function 'IntegrateRange'

    IMPLICIT NONE
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: lambda, flux
    REAL(KIND=8), INTENT(IN) :: llim, ulim
    REAL(KIND=8) :: del

    IF (ulim <= llim) THEN
       del = 1.
    ELSE
       del = ulim - llim
    END IF

    AverageRange = IntegrateRange(lambda, flux, llim, ulim) / del

  END FUNCTION AverageRange


  REAL(KIND=8) FUNCTION interplinlin(x1, x2, y1, y2, t)
    
    IMPLICIT NONE
    REAL(KIND=8) :: x1, x2, y1, y2, t
    
    interplinlin = y1 + (y2 - y1) * (t - x1) / (x2 - x1)
    
  END FUNCTION interplinlin


  REAL(KIND=8) FUNCTION IntegrateHisto(lambda, flux)

    ! Integrates histogram

    IMPLICIT NONE
    REAL(KIND=8), DIMENSION(:), INTENT(IN) :: lambda, flux
    INTEGER :: imax, i
    REAL(KIND=8) :: dlambda

    imax = MIN(SIZE(lambda), SIZE(flux))
    IF (imax < SIZE(lambda)) PRINT*, &
         'IntegrateHisto: More wavelengths than fluxes!'
    IF (imax > SIZE(lambda)) PRINT*, &
         'IntegrateHisto: Less wavelengths than fluxes!' 

    IntegrateHisto = 0

    DO i = 1, imax

       IF (i == 1) THEN
          dlambda = lambda(i+1) - lambda(i)
       ELSE IF (i == imax) THEN
          dlambda = lambda(i) - lambda(i-1)
       ELSE
          dlambda = (lambda(i+1) - lambda(i-1)) / 2
       END IF

       IntegrateHisto = IntegrateHisto + dlambda * flux(i)

    END DO

  END FUNCTION IntegrateHisto


end subroutine stellpop


END module mcrecsp
