module readcompphotfnu_mod
use ParamDef
contains

subroutine readcompphotfnu(nz,z,fluxfilt, mmax, m_IR, minlam, area,trans,lamfilt,&
     typetrans,mnlamfilt, nfilters,ifilt,meanlam,H0,Omega_M,Omega_L,zacc,relerr,systerr)



!OUTPUT (AA)
INTEGER, INTENT(OUT) :: nz, nfilters, mmax,m_IR
REAL(KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: z, fluxfilt , area,meanlam
REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE, INTENT(OUT) :: trans, lamfilt
REAL(KIND=8), INTENT(OUT):: minlam
INTEGER, DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: mnlamfilt, typetrans, ifilt
real(KIND=8), INTENT(OUT) :: H0,Omega_M,Omega_L,relerr,systerr
integer, INTENT(OUT) :: zacc
!END O (AA)

INTEGER, PARAMETER :: nmaxlamfilt = 1500 ! Check 'filterfile'!
CHARACTER(LEN=*), PARAMETER :: filenames = 'cigale.par' ! INPUT! 
CHARACTER(LEN=*), PARAMETER :: cosmofile = 'cosmology.dat' ! INPUT! 
CHARACTER(LEN=80) ::  filterfile, calibfile, filtoutpar, specfile
CHARACTER(LEN=80) ::  zdata, photdat, bayespar, myfile1,myfile2,outfilelist
INTEGER :: status_open, status_read
LOGICAL :: fl_zdata
CHARACTER(LEN=32) :: ID
CHARACTER(LEN=160) :: header
CHARACTER(LEN=1) :: sharp
INTEGER:: idummy, i, h, m,nval, nfilt, nobj
INTEGER:: lspace, rspace, iz, minz
INTEGER, DIMENSION(:), ALLOCATABLE :: nlamfilt, intz, sintz
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: tempz, fnu
CHARACTER(LEN=8) :: cdummy
REAL(KIND=8) :: rdummy
CHARACTER(LEN=8), DIMENSION(:), ALLOCATABLE :: filtlab
REAL(KIND=8), DIMENSION(3) :: zpar
CHARACTER(LEN=800) :: fmt
CHARACTER(LEN=80) :: infilelist

OPEN (10, STATUS='OLD', FILE=filenames, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, 'Parameter file ', filenames, ' not present!'
   STOP
END IF

DO h = 1, 16
   SELECT CASE (h)
   CASE (2)
      READ (10, '(A)') filterfile
      filterfile = TRIM(filterfile) // '.dat'
   CASE (4)
      READ (10, '(A)') calibfile
      calibfile = TRIM(calibfile) // '.dat'
  CASE (6)
      READ (10, '(A)') filtoutpar
      filtoutpar = TRIM(filtoutpar) // '.par'
   CASE (8)
      READ (10, '(A)') zdata
   CASE (10)
      READ (10, '(A)') photdat
      IF (zdata == photdat) THEN
         fl_zdata = .TRUE.
         zdata = TRIM(zdata) // '.fnu'
      ELSE
         fl_zdata = .FALSE.
         zdata = TRIM(zdata) // '.dat'
      END IF                                                                 
   CASE (12)
   READ (10, *) zacc
   CASE(14)
   READ (10, *) relerr
   CASE(16)
      READ (10, *) systerr              

   CASE DEFAULT
      READ (10, '(A)') header
      IF (header(1:1) /= '#') PRINT*, 'Wrong header !'
  
END SELECT
END DO
      CLOSE(10)


! *** Read the filter transmissions ***               
OPEN (20, STATUS='OLD', FILE=filterfile, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, TRIM(filterfile), ' is missing!'
   STOP
END IF
                        
READ (20, '(A)') header
READ (20, *) sharp, nfilters
READ (20, '(A)') header
READ (20, '(A)') hseader

ALLOCATE (nlamfilt(nfilters), typetrans(nfilters))
ALLOCATE (lamfilt(nfilters,nmaxlamfilt))
ALLOCATE (trans(nfilters,nmaxlamfilt))
DO i = 1, nfilters
   READ (20, *) nlamfilt(i), typetrans(i)
   READ (20, *) (lamfilt(i,j), trans(i,j), j = 1, nlamfilt(i))
   IF (nlamfilt(i) > nmaxlamfilt) &
        PRINT*, 'Warning: Filter ', i, 'not completely read!'
END DO

CLOSE (20)

! *** Read the filter calibrations ***                    
ALLOCATE (area(nfilters), meanlam(nfilters))

OPEN (30, STATUS='OLD', FILE=calibfile, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, TRIM(calibfile), ' is missing!'
   STOP
END IF

READ (30, '(A)') header
DO i = 1, nfilters
   READ (30, *) idummy, cdummy, rdummy, area(i), meanlam(i)
END DO

CLOSE(30)
! *** Read output information for filter fluxes ***          
ALLOCATE (ifilt(nfilters))
ALLOCATE (filtlab(nfilters))
filtlab = ' '

OPEN (40, STATUS='OLD', FILE=filtoutpar, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, TRIM(filtoutpar), ' is missing!'
   STOP
END IF

DO
   READ (40, '(A)', IOSTAT=status_read) header
   IF (header(1:1) /= '#') EXIT
END DO
BACKSPACE (40)

m = 0
m_IR = 0
DO
   m = m + 1
   READ (40, *, IOSTAT=status_read) &
        ifilt(m), filtlab(m), rdummy, cdummy
   IF (status_read /= 0 .OR. filtlab(m) == ' ') EXIT
   IF (ifilt(m) == 0) m_IR = m
END DO
mmax = m - 1

CLOSE (40)

! Mark irrelevant filters                                                                                                                       
ALLOCATE (mnlamfilt(nfilters))
mnlamfilt = 0
DO m = 1, mmax
   mnlamfilt(ifilt(m)) = nlamfilt(ifilt(m))
END DO

! Minimum wavelength of relevant filters (for IGM correction)                                                                                   
minlam = MINVAL(lamfilt(:,1), MASK = mnlamfilt /= 0)

! *** Read redshifts ***                                  
! Consider different cases:                                
! - 3 parameters in a row in '*.dat' (z_start, z_end, z_step): fl_zdata=.false.
! - List of redshifts (1 column in '*.dat'): fl_zdata=.false.                  
! - Redshifts from photometric data file (column 2 in '*.fnu': fl_zdata=.true. 
! - Missing files and wrong format -> z = 0                                    

OPEN (60, STATUS='OLD', FILE=zdata, IOSTAT=status_open)

IF (status_open /= 0) THEN

   PRINT*, TRIM(zdata), ' is missing! -> Assume z = 0!'
   nz = 1
   ALLOCATE (z(1:nz))
   z(1) = 0.
ELSE

   IF (fl_zdata) THEN
      DO h = 1, 4
         IF (h == 2) THEN
            READ (60, *, IOSTAT=status_read) sharp, nval, nfilt, nobj
       
            IF (nfilt /= mmax) &
                 PRINT*, 'Inconsistent number of filters in ', TRIM(zdata)
         ELSE
            READ (60, '(A)', IOSTAT=status_read) header

      !     PRINT *, "z from *.fnu: ", header                   
                                                       
         END IF
      END DO
   ELSE
      DO
         READ (60, '(A)', IOSTAT=status_read) header
         IF (header(1:1) /= '#') EXIT
!         PRINT *, "z from *.dat: ", header                 

        
      END DO
   ENDIF
   BACKSPACE (60)

   lspace = INDEX(TRIM(ADJUSTL(header)), ' ', .FALSE.)
   rspace = INDEX(TRIM(ADJUSTL(header)), ' ', .TRUE.)
   IF (fl_zdata) THEN

      IF (lspace == rspace) THEN

         PRINT*, TRIM(zdata), &
              ' does not have required format -> Assume z = 0!'
         nz = 1
         ALLOCATE (z(nz))
         z(1) = 0.

       
      ELSE

         ALLOCATE (tempz(nobj))
         ALLOCATE (intz(nobj), sintz(nobj))

         DO iz = 1, nobj
            READ (60, *, IOSTAT=status_read) ID, tempz(iz)
         
            IF (status_read /= 0) EXIT
         END DO
         intz = NINT(tempz * 10. ** zacc)

         iz = 0
         minz = -1
         DO
            iz = iz + 1
            sintz(iz) = MINVAL(intz, MASK = intz > minz)
            minz = sintz(iz)
            IF (minz == MAXVAL(intz)) EXIT
         END DO
         nz = iz
!         PRINT *, "nz = ",nz                                   

         ALLOCATE (z(nz))
         z = REAL(sintz(1:nz), 8) / 10. ** zacc
   !      PRINT *, "redshifts: ", z                                   
         DEALLOCATE (tempz, intz, sintz)

      END IF

   ELSE

      IF (lspace /= rspace) THEN

         READ (60, *) zpar
         nz = FLOOR((zpar(2) - zpar(1)) / zpar(3) + tol) + 1
         ALLOCATE (z(nz))
         z = (/(zpar(1) + (iz - 1) * zpar(3), iz = 1, nz)/)

      ELSE IF (lspace == rspace .AND. lspace /= 0) THEN

         PRINT*, 'Step size is missing! -> Assume z_step = z_end - z_start!'
         nz = 2
         ALLOCATE (z(nz))
         READ (60, *) z(1), z(2)
!         PRINT *, z(1), " < z < ", z(2)               

      ELSE

         ALLOCATE (tempz(nzmax))

         iz = 0
         DO
            READ (60, *, IOSTAT=status_read) tempz(iz+1)
            IF (status_read /= 0) EXIT
            iz = iz + 1
         END DO
         nz = iz

         ALLOCATE (z(nz))
         z = tempz(1:nz)
         DEALLOCATE (tempz)

      END IF

   END IF

   CLOSE (60)

END IF


! *** Read cosmological parameters ***

OPEN (70, STATUS='OLD', FILE=cosmofile, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, TRIM(cosmofile), ' is missing!'
   STOP
END IF!

READ (70, '(A)') header
READ (70, *) H0, Omega_M, Omega_L

CLOSE (70)







ALLOCATE (fluxfilt(nfilters))








CONTAINS 

  

  CHARACTER(LEN=8) FUNCTION IntToChar(intnum, nchar)
    
    ! Converts integer numbers (8 digits in maximum) to character strings 

    IMPLICIT NONE
    INTEGER, INTENT(IN) :: intnum, nchar
    REAL(KIND=8) :: rnum
    REAL(KIND=8), PARAMETER :: tol = 1d-12
    INTEGER :: absint, j, i, inum
    CHARACTER(LEN=nchar) :: cnum

    absint = ABS(intnum)
    rnum = (REAL(absint, 8) + tol) / 10. ** (nchar - 1)
 
    j = 0
    DO WHILE (rnum >= 10)
       rnum = rnum / 10
       j = j + 1
    END DO
    IF (j >= 1) PRINT*, '(AI1A)', 'IntToChar: Integer too long -> last ', j, &
         ' digits not considered'

    DO i = 1, nchar
       inum = INT(rnum) 
       cnum(i:i) = ACHAR(48 + inum)  
       rnum = ((rnum - REAL(inum, 8)) * 10.)
    END DO
     
    IntToChar = TRIM(cnum)
    
  END FUNCTION IntToChar



end subroutine readcompphotfnu

end module readcompphotfnu_mod
