module readeffdustdh_mod

contains

subroutine readeffdustdh(nirmod,irid,irslope,nirlam,irflux,irlam,nhdlam,&
     hdlam,hdflux,nlambdalin,lambdalin,emlin,abslin,nrange,lambdarange)

use ParamDef

!OUTPUT (AA)
CHARACTER(LEN=2), DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: irid
REAL(KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: irslope, hdlam,lambdalin
REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE, INTENT(OUT) :: irlam, irflux,& 
     hdflux,emlin, abslin, lambdarange
INTEGER, INTENT(OUT) :: nirmod, nirlam, nhdlam, nlambdalin, nrange

!END O (AA)

CHARACTER(LEN=32), DIMENSION(:), ALLOCATABLE :: agntempl
CHARACTER(LEN=*), PARAMETER :: indir = 'inspec/' ! INPUT folder!
CHARACTER(LEN=*), PARAMETER :: prefix = 'e' ! OUTPUT!
CHARACTER(LEN=80) :: irmodpar, linpar, ircaltab, irmod, cdummy, interpoldat, specin
CHARACTER(LEN=1) :: flrelpar, sharp
CHARACTER(LEN=160) :: header,myfile1,myfile2,fmt, fmtlam, fmtflux
CHARACTER(LEN=16) :: irlab
INTEGER :: nalpha, nagn, nfagn, nal, nat, nfa, m, i, n
INTEGER :: status_open, status_read, ndir
REAL(KIND=8) :: lamsep, rdummy, alpha0, fl_agn, fagn0, Lbol0, ffuv0, fv0
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: fagn
INTEGER, DIMENSION(2) :: ntempl
CHARACTER(LEN=16), DIMENSION(:), ALLOCATABLE :: emlinname, abslinname
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: lambdatest
CHARACTER(LEN=80) :: infilelist, outfilelist, outdir, spec, specout
INTEGER, PARAMETER :: ncol = 6 ! OUTPUT!

ircaltab = indir // 'dhcal' // '.dat'
irlab='irdh'
nfagn=1
nagn=1
ALLOCATE (agntempl(nagn))
agntempl(1)='agnx_00125_1200_032'
agntempl = indir // agntempl
myfile1=trim(rootname)//'crecsp' 
infilelist = TRIM(myfile1) // '.list'
myfile2=trim(rootname)//'attspec'
outfilelist = TRIM(myfile2) // '.list'
 
! *** Read IR model calibrations ***

IF (TRIM(irmodpar) /= '_') THEN


   OPEN (24, STATUS='OLD', FILE=ircaltab, IOSTAT=status_open)
   IF (status_open /= 0) THEN
      PRINT*, 'Data file ', TRIM(ircaltab), ' not present!'
      STOP
   END IF

   READ (24, '(A)') header
   READ (24, *) sharp, nirmod
   ALLOCATE (irid(nirmod), irslope(nirmod))
   READ (24, '(A)') header
   READ (24, *) (irid(m), irslope(m), rdummy, rdummy, rdummy, m = 1, nirmod)
   
   CLOSE (24)

END IF


! *** Read IR models ***

IF (TRIM(irmodpar) /= '_') THEN

   DO m = 1, nirmod

      irmod = indir // TRIM(irlab) // irid(m) // '.spec'

      OPEN (26, STATUS='OLD', FILE=irmod, IOSTAT=status_open)
      IF (status_open /= 0) THEN
         PRINT*, 'File ', TRIM(irmod), ' not present!'
         STOP
      END IF

      READ (26, *) nirlam
      IF (m == 1) ALLOCATE (irlam(nirmod,nirlam), irflux(nirmod,nirlam))
      READ (26, *) (irlam(m,i), irflux(m,i), i = 1, nirlam)

      CLOSE (26)

   END DO

END IF


! *** Read AGN templates ***

IF (TRIM(irmodpar) /= '_') THEN

   DO nat = 1, nagn

         OPEN (28, STATUS='OLD', FILE=TRIM(agntempl(nat))//'.spec', &
              IOSTAT=status_open)
         IF (status_open /= 0) THEN
            PRINT*, 'File ', TRIM(agntempl(nat)) // '.spec not present!'
            STOP
         END IF

         READ (28, *) nhdlam
         ALLOCATE (hdlam(nhdlam))
         IF (nat == 1) ALLOCATE (hdflux(nagn, nhdlam))
         READ (28, *) (hdlam(i), hdflux(nat, i), i = 1, nhdlam)

         CLOSE (28)

         DO m = 1, nirmod
            IF (.NOT. ALL(irlam(m,:) == hdlam)) PRINT*, &
                 'Wavelengths of IR model', m, 'and the AGN template', nat, &
                 'do not match!'
         END DO

         DEALLOCATE (hdlam)

      END DO

END IF


! *** Read line template parameters ***

ntempl(1)=2
ntempl(2)=2
!lim(1,1)=3400
!lim(1,2)=3600
!lim(2,1)=1420
!lim(2,2)=1490
ALLOCATE (emlinname(ntempl(1)))
emlinname(1)="emlin1"
emlinname(2)="emlin2"
emlinname = indir // emlinname
!ratem(1)=1.
!ratem(2)=3.7
!ALLOCATE (ratem(ntempl(1)))
ALLOCATE (abslinname(ntempl(2)))
abslinname(1)="uvlin1"
abslinname(2)="uvlin2"
abslinname = indir // abslinname
!ratabs(1)=1.
!ratabs(2)=3.7
interpoldat="lambdainterpol"
interpoldat = indir // interpoldat
!limrl(1)=0.3
!limrL(2)=0.1


! *** Read line templates ***

! Emission lines

IF (ntempl(1) /= 0) THEN
   DO n = 1, ntempl(1)
      OPEN (40, STATUS='OLD', FILE=TRIM(emlinname(n))//'.dat', &
           IOSTAT=status_open)
      IF (status_open /= 0) THEN
         PRINT*, 'Template file ', TRIM(emlinname(n)) // '.dat not present!'
         STOP
      END IF
      READ (40, *) nlambdalin
      IF (n == 1) THEN 
         ALLOCATE (emlin(ntempl(1),nlambdalin))
         ALLOCATE (lambdalin(nlambdalin), lambdatest(nlambdalin))
      END IF
      IF (n > 1) lambdatest = lambdalin
      READ (40, *) (lambdalin(i), emlin(n,i), i = 1, nlambdalin) 
      IF (n > 1 .AND. .NOT.ALL(lambdalin == lambdatest)) &
           PRINT*, 'Wavelengths of template files do not match!'
      CLOSE (40)
   END DO
ELSE
   ALLOCATE (emlin(ntempl(1),0))
END IF

! Absorption lines

IF (ntempl(2) /= 0) THEN
   DO n = 1, ntempl(2)
      OPEN (40, STATUS='OLD', FILE=TRIM(abslinname(n))//'.dat', &
           IOSTAT=status_open)
      IF (status_open /= 0) THEN
         PRINT*, 'Template file ', TRIM(abslinname(n)) // '.dat not present!'
         STOP
      END IF
      READ (40, *) nlambdalin
      IF (n == 1) THEN 
         ALLOCATE (abslin(ntempl(2),nlambdalin))
         IF (ntempl(2) == 0) &
              ALLOCATE (lambdalin(nlambdalin), lambdatest(nlambdalin))
      END IF
      IF (n > 1 .OR. ntempl(2) /= 0) lambdatest = lambdalin
      READ (40, *) (lambdalin(i), abslin(n,i), i = 1, nlambdalin) 
      IF ((n > 1 .OR. ntempl(2) /= 0) .AND. .NOT.ALL(lambdalin==lambdatest)) &
           PRINT*, 'Wavelengths of template files do not match!'
      CLOSE (40)
   END DO
ELSE
   ALLOCATE (abslin(ntempl(2),0))  
END IF

! Wavelength list for interpolation of model lines

OPEN (40, STATUS='OLD', FILE=TRIM(interpoldat)//'.dat', IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, 'Data file ', TRIM(interpoldat)//'.dat', ' not present!'
   STOP
END IF
READ (40, *) nrange
ALLOCATE (lambdarange(2,nrange)) 
IF (nrange > 0) &
     READ (40, *) (lambdarange(1,i), lambdarange(2,i), i = 1, nrange)
CLOSE (40)



end subroutine readeffdustdh

end module readeffdustdh_mod
