
module readmcrecsp_mod

contains

subroutine readmcrecsp(age,mstar,flux,dage,d4000,lambda,nage,iagemax,metall)



INTEGER, PARAMETER :: agemax = 67, lambdamax = 1221, nv = 1
! OUTPUT (AA)

REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE, INTENT(OUT) :: age, mstar
REAL(KIND=8), DIMENSION(:,:,:), ALLOCATABLE,INTENT(OUT) :: flux
REAL(KIND=8), DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: dage, d4000
REAL(KIND=8), DIMENSION(lambdamax),INTENT(OUT) :: lambda
INTEGER, INTENT(OUT) :: nage
INTEGER, DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: iagemax
! END O (AA)

REAL(KIND=8) metall
REAL(KIND=8), DIMENSION(:), ALLOCATABLE :: met
CHARACTER(LEN=*), PARAMETER :: indir = 'inspec/' ! INPUT folder!
INTEGER :: nm,i,j,iage
CHARACTER(LEN=8) :: metc
CHARACTER(LEN=80) :: idin, specin, massin, d4000dat,header
CHARACTER(LEN=1) :: imf = 's'
REAL(KIND=8) :: rdummy
INTEGER :: status_read, status_open


ALLOCATE (age(nv,agemax))
ALLOCATE (flux(nv,agemax,lambdamax))
ALLOCATE (iagemax(nv))
ALLOCATE (mstar(nv,agemax))
ALLOCATE (met(1))

nm=1
met(nm)=metall!0.02




metc = IntToChar(20,2) !IntToChar(NINT(met(nm) * 1000), 2)
idin = TRIM(indir) // 'm' // imf // 'z' // TRIM(metc)

! Read SSP models

specin = TRIM(idin) // '.spec'
PRINT*, 'READ ', TRIM(specin)
   
OPEN (30, STATUS='OLD', FILE=specin, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, 'File ', TRIM(specin), ' not present!'
   STOP
END IF
      
DO i = 1, agemax
   DO j = 1, lambdamax
      READ (30, *, IOSTAT=status_read) &
           age(nm,i), rdummy, lambda(j), flux(nm,i,j)
   END DO
   IF (status_read /= 0) EXIT
END DO

iagemax(nm) = i - 1
IF (iagemax(nm) < agemax) PRINT*, 'Warning: Less ages than expected!'

CLOSE (30)


! Read stellar-mass tables

massin = TRIM(idin) // '.mass'
      
OPEN (40, STATUS='OLD', FILE=massin, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, 'File ', TRIM(massin), ' not present!'
   STOP
END IF
      
READ (40, '(A)') header
READ (40, '(A)') header

DO i = 1, agemax
   READ (40, *, IOSTAT=status_read) rdummy, rdummy, mstar(nm,i)
   IF (status_read /= 0) EXIT
END DO
   
IF (i /= iagemax(nm) + 1) THEN
   PRINT*, 'Ages of SSP spectra and mass table do not match!'
   STOP
END IF

CLOSE (40)



! *** Read D4000 data for single stellar populations *** 

! Aim: Derivation of D4000-related galaxy age
! Calibration: Balogh et al. (1999)

d4000dat = &
     TRIM(indir) // 'd4000_m' // TRIM(imf) // 'z' // TRIM(metc) // 'v.dat'

OPEN (60, STATUS='OLD', FILE=d4000dat, IOSTAT=status_open)
IF (status_open /= 0) THEN
   PRINT*, TRIM(d4000dat), ' is missing!'
   STOP
END IF

READ (60, *) nage
ALLOCATE (dage(nage), d4000(nage))

DO iage = 1, nage
   READ (60, *) dage(iage), d4000(iage) 
END DO

CLOSE (60)



CONTAINS

  CHARACTER(LEN=8) FUNCTION IntToChar(intnum, nchar)
    
    ! Converts integer numbers (8 digits in maximum) to character strings 

    IMPLICIT NONE
    INTEGER, INTENT(IN) :: intnum, nchar
    REAL(KIND=8) :: rnum
    REAL(KIND=8), PARAMETER :: tol = 1d-12
    INTEGER :: absint, j, i, inum
    CHARACTER(LEN=nchar) :: cnum
  
    absint = ABS(intnum)
    rnum = (REAL(absint, 8) + tol) / 10. ** (nchar - 1)
    
    j = 0
    DO WHILE (rnum >= 10)
       rnum = rnum / 10
       j = j + 1
    END DO
    IF (j >= 1) PRINT*, '(AI1A)', 'IntToChar: Integer too long -> last ', j, &
         ' digits not considered'
  
    DO i = 1, nchar
       inum = INT(rnum) 
       cnum(i:i) = ACHAR(48 + inum)  
       rnum = ((rnum - REAL(inum, 8)) * 10.)
    END DO
     
    IntToChar = TRIM(cnum)
    
  END FUNCTION IntToChar

END SUBROUTINE readmcrecsp

end module readmcrecsp_mod
